// BlockedUsersListDlg.cpp : implementation file
//

#include "stdafx.h"
#include "MessengerClient.h"
#include "BlockedUsersListDlg.h"
#include "MessengerClientDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBlockedUsersListDlg dialog


CBlockedUsersListDlg::CBlockedUsersListDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBlockedUsersListDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CBlockedUsersListDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


CBlockedUsersListDlg::CBlockedUsersListDlg(CMessengerClientDoc *pDoc, CWnd* pParent /*=NULL*/)
	: CDialog(CBlockedUsersListDlg::IDD, pParent)
{
	m_pParent = pDoc;
}


void CBlockedUsersListDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBlockedUsersListDlg)
	DDX_Control(pDX, IDOK, m_btnUnblock);
	DDX_Control(pDX, IDC_BLOCKED_USERS_LIST, m_BlockedUsersList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CBlockedUsersListDlg, CDialog)
	//{{AFX_MSG_MAP(CBlockedUsersListDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBlockedUsersListDlg message handlers

/**************************************************************************/
// Method Name	: OnInitDialog 
// Description	: Called when dialog is being initialized
// Inputs		: None
// Output		: Reads the blocked users list from doc object and fills 
//					those names in the list box
// Returns		: void
/**************************************************************************/
BOOL CBlockedUsersListDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// See is this user has blocked any other users
	// If list is not obtained or is empty, no need to go further
	if(m_pParent->m_BlockedUsersList.IsEmpty())
		return TRUE;

	// Get addresses of head and tail members of the list
	CMessagePacket *pHeadMsg = (CMessagePacket *)m_pParent->m_BlockedUsersList.GetHead();
	CMessagePacket *pTailMsg = (CMessagePacket *)m_pParent->m_BlockedUsersList.GetTail();
	CMessagePacket *pNextMsg = NULL;

	// Store the head position
	POSITION pos = m_pParent->m_BlockedUsersList.GetHeadPosition();
	pHeadMsg = (CMessagePacket *)m_pParent->m_BlockedUsersList.GetNext(pos);

	while(1)
	{
		if(pHeadMsg != pTailMsg)
			pNextMsg = (CMessagePacket *)m_pParent->m_BlockedUsersList.GetNext(pos);
		
		// Put this name into the listbox
		m_BlockedUsersList.AddString(pHeadMsg->m_strFrom);

		// Enable the Unblock Sender button
		m_btnUnblock.EnableWindow();
		
		// If we ve reached end of list
		if(pHeadMsg == pTailMsg)
			break;

		// Else go ahead 1 position
		pHeadMsg = pNextMsg;
	}

	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}



/**************************************************************************/
// Method Name	: OnOK 
// Description	: Called when Unblock Selected button is pressed
// Inputs		: None
// Output		: Creates a message packet with reason flag = "
// Returns		: void
/**************************************************************************/
void CBlockedUsersListDlg::OnOK() 
{
	// If nothing is selected
	int cnt = m_BlockedUsersList.GetSelCount();
	if(!cnt)
	{
		::AfxMessageBox("Please select at least 1 user");
		return;
	}

	// Prepare a message packet
	CMessagePacket msg;
	msg.m_strFrom = m_pParent->m_UserName;
	msg.m_strReasonFlag = "BUR";		// Blocked Users Removal

	int *selArr = new int[cnt];
	m_BlockedUsersList.GetSelItems(cnt, selArr);

	// Put the selected names into the msg list of message packet
	CString str;
	for(int i = 0; i < cnt; i++)
	{
		// Get the selected name into str
		m_BlockedUsersList.GetText(selArr[i], str);

		// Put str into msg list
		msg.m_msgList.AddTail(str);
	}

	// Free the memory aquired by selArr
	delete[] selArr;

	// Send this msg packet to server
	m_pParent->SendMessagePacket(msg);

	// Hide this window. It will be visible only when some response from server
	// is obtained
	this->ShowWindow(SW_HIDE);

	//CDialog::OnOK();
}



/**************************************************************************/
// Method Name	: OnCancel 
// Description	: Called when Cancel button is pressed
// Inputs		: None
// Output		: Frees the memory allocated for this dialog
// Returns		: void
/**************************************************************************/
void CBlockedUsersListDlg::OnCancel() 
{
	CDialog::OnCancel();

	m_pParent->m_pBlockedUsersDlg = NULL;
	delete(m_pParent->m_pBlockedUsersDlg);
}
