// ChangePassword.cpp : implementation file
//

#include "stdafx.h"
#include "MessengerClient.h"
#include "ChangePassword.h"
#include "MessengerClientDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CChangePassword dialog


CChangePassword::CChangePassword(CWnd* pParent /*=NULL*/)
	: CDialog(CChangePassword::IDD, pParent)
{
	//{{AFX_DATA_INIT(CChangePassword)
	m_OldPassword = _T("");
	m_Password = _T("");
	m_Password2 = _T("");
	//}}AFX_DATA_INIT
}

CChangePassword::CChangePassword(CMessengerClientDoc *pDoc, CWnd* pParent /*= NULL*/)   // additional constructor
	: CDialog(CChangePassword::IDD, pParent)
{
	m_OldPassword = _T("");
	m_Password = _T("");
	m_Password2 = _T("");
	m_pParent = pDoc;
}

void CChangePassword::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CChangePassword)
	DDX_Text(pDX, IDC_OLD_PASSWORD, m_OldPassword);
	DDX_Text(pDX, IDC_PASSWORD, m_Password);
	DDX_Text(pDX, IDC_PASSWORD2, m_Password2);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CChangePassword, CDialog)
	//{{AFX_MSG_MAP(CChangePassword)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



/**************************************************************************/
// Method Name	: IsPasswordValid
// Description	: This method verifies the 2 password entries typed by the user
// Inputs		: Both entries made by the user in 'password' and 'retype password'
//				  edit boxes
// Output		: None
// Returns		: 1	- Password entris don't match
//				  2 - Password contains spaces
//				  3 - Password is too long or null
//				  4 - Password contains invalid chars (anything except A-Za-z0-9)
//				  0 - Everything is all right
// Note			: When making changes to this method, also take care to make similar
//				  changes in CMessengerDialog::IsPasswordValid and
//				  CRegisterDlg::IsPasswordValid methods
/**************************************************************************/
int CChangePassword::IsPasswordValid(CString password, CString password2 /* = NULL*/)
{
	if(password2 != "")		// We are checking password, not login
	{
		// Check whether the password entries match
		if(password != password2)
			return 1;
	}

	// Eliminate the spaces on left and right of password
	CString tstPass = password;
	tstPass.TrimLeft();
	tstPass.TrimRight(' ');

	if(tstPass != password)		// Password contains spaces on left and/or right
		return 2;
	
	// If password is more than MAX_PASSWORD_LENGTH chars long or zero
	int len = password.GetLength();
	if(len == 0 || len > MAX_PASSWORD_LENGTH)
		return 3;


	// If password contains anything except A-Z, a-z and 0-9
	for(int i = 0; i < len; i++)
	{
		char ch = password.GetAt(i);
		if((ch < 'A' || ch > 'Z') && (ch < 'a' || ch > 'z') && (ch < '0' || ch > '9'))
			return 4;
	}

	return 0;
}


/////////////////////////////////////////////////////////////////////////////
// CChangePassword message handlers




/**************************************************************************/
// Method Name	: OnOK
// Description	: Called when user clicks 'Change Password' button
// Inputs		: None
// Output		: Sends the password change request to the server
//				  If password is invalid, shows appropriate message to the user
// Returns		: void
/**************************************************************************/
void CChangePassword::OnOK() 
{
	CMessagePacket m_MsgPacket;

	// Get values from screen
	UpdateData(TRUE);

	int ret = IsPasswordValid(m_OldPassword, "");

	// If old password in entered invalid
	if(ret)
	{
		::AfxMessageBox("Old password is invalid, Please try again");
		return;
	}

	int ret1 = IsPasswordValid(m_Password, m_Password2);

	switch(ret1)
	{
	case 0:				// Password is all right

		// Create and initialize a message packet
		m_MsgPacket.m_strFrom = m_pParent->m_UserName;
		m_MsgPacket.m_strTo = m_Password;
		m_MsgPacket.m_strMessageBody = m_OldPassword;
		m_MsgPacket.m_strReasonFlag = "C";	// Because we are changing password in here

		// Send some data on the connected socket
		m_pParent->SendMessagePacket(m_MsgPacket);
		break;

	case 1:				// password entries do not match
		::AfxMessageBox("Password entries much match");
		break;

	case 2:				// Password contains spaces
		::AfxMessageBox("Password cannot contain spaces");
		break;

	case 3:				// Password length incorrect
		::AfxMessageBox("Password length must be 1-14 characters");
		break;

	case 4:
		::AfxMessageBox("Password can contain only the characters from A-Z, a-z and 0-9");
		break;
	}

	//CDialog::OnOK();
}




/**************************************************************************/
// Method Name	: OnCancel
// Description	: Called when user clicks 'Cancel' button
// Inputs		: None
// Output		: Closes the change password dialog and nullifies the 
//				  password dialog pointer in doc class
// Returns		: void
/**************************************************************************/
void CChangePassword::OnCancel() 
{
	m_pParent->m_pChangePasswordDlg = NULL;
	
	CDialog::OnCancel();
}
