// ColorFontsPreferences.cpp : implementation file
//

#include "stdafx.h"
#include "MessengerClient.h"
#include "ColorFontsPreferences.h"
#include "MessengerClientDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CColorFontsPreferences property page

IMPLEMENT_DYNCREATE(CColorFontsPreferences, CPropertyPage)

CColorFontsPreferences::CColorFontsPreferences() : CPropertyPage(CColorFontsPreferences::IDD)
{
	//{{AFX_DATA_INIT(CColorFontsPreferences)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	bDirty = false;
}

CColorFontsPreferences::~CColorFontsPreferences()
{
}

void CColorFontsPreferences::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CColorFontsPreferences)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CColorFontsPreferences, CPropertyPage)
	//{{AFX_MSG_MAP(CColorFontsPreferences)
	ON_BN_CLICKED(IDC_COLOR, OnColor)
	ON_BN_CLICKED(IDC_FONT, OnFont)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CColorFontsPreferences message handlers




/**************************************************************************/
// Method Name	: OnColor
// Description	: Called when user clicks 'Choose Color' button in Preferences
//				  dialog
// Inputs		: None
// Output		: Shows a color selection dialog
//				  If user clicks ok,
//				  Sets the color chosen by user in preferences
// Returns		: TRUE - Success
//				  FALSE - otherwise
/**************************************************************************/
void CColorFontsPreferences::OnColor() 
{
	CColorDialog 	colorDialog;

	// If user chose a color
	if(colorDialog.DoModal() == IDOK)
	{
		// Find out the color user chose
		COLORREF color = colorDialog.GetColor();

		// Put this color into the property sheet
		m_pParent->m_pPreferencesDlg->m_Preferences.m_iColor = color;

		// Set the modified flag to true.
		SetModified();
		bDirty = true;
	}
}



/**************************************************************************/
// Method Name	: OnFont
// Description	: Called when user clicks 'Choose Font' button in Preferences
//				  dialog
// Inputs		: None
// Output		: Shows a font selection dialog
//				  If user clicks ok,
//				  Validates whether usre has chosen a font.
//				  Sets the font chosen by user in preferences, and also its color
//				  and size.
// Returns		: TRUE - Success
//				  FALSE - otherwise
/**************************************************************************/
void CColorFontsPreferences::OnFont() 
{
	CFontDialog 	fontDialog;

	// If user chose a color
	if(fontDialog.DoModal() == IDOK)
	{
		// Find out the font user chose
		LOGFONT font;
		fontDialog.GetCurrentFont(&font);
		COLORREF fontColor = fontDialog.GetColor();

		// If user did not choose any font, no need to process it
		if(!strcmp(font.lfFaceName, ""))
			return;

		// Put this font into property sheet
		m_pParent->m_pPreferencesDlg->m_Preferences.m_Font = font;

		m_pParent->m_pPreferencesDlg->m_Preferences.m_iFontColor = fontColor;
		m_pParent->m_pPreferencesDlg->m_Preferences.m_iFontSize = fontDialog.GetSize() * 2;

		// Set the modified flag to true.
		SetModified();
		bDirty = true;
	}
}



/**************************************************************************/
// Method Name	: OnApply
// Description	: Called when user clicks 'Apply' or 'OK' button in Preferences
//				  dialog
// Inputs		: None
// Output		: Overwrites preferences with dialog settings
//				  Changes the messenger client's appearance accordingly.
//				  Calls base class method
// Returns		: TRUE - Success
//				  FALSE - otherwise
/**************************************************************************/
BOOL CColorFontsPreferences::OnApply() 
{
	if(bDirty)
	{
		// Copy all preferences from preferences property sheet to doc object
		m_pParent->m_Preferences = m_pParent->m_pPreferencesDlg->m_Preferences;

		// Call the doc class's apply function
		m_pParent->ApplyPreferenceChanges(COlOR_FONTS_PREFERENCES);
	}
	
	return CPropertyPage::OnApply();
}
