//////////////////////////////////////////////////////////////////////
// CommonDialogElements.cpp: 
// implementation of the CCommonDialogElements class.
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CommonDialogElements.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCommonDialogElements::CCommonDialogElements()
{
	m_pSocket	= NULL;
	m_pArIn		= NULL;
	m_pArOut	= NULL;
	m_pSocketFile = NULL;
	m_pMsgPacket= NULL;
}

CCommonDialogElements::~CCommonDialogElements()
{
	delete m_pSocket;
	delete m_pSocketFile;
}



/**************************************************************************/
// Method Name	: Connect2ServerSocket()
// Description	: Used to connect to the server socket and
//				  create various archive objects
// Inputs		: None
// Output		: Opens config file, reads data
//				  Connects to server socket and creates archives for data
//				  exchange
//				  Creates a message packet
// Returns		: TRUE if successful, otherwise FALSE
/**************************************************************************/
BOOL CCommonDialogElements::Connect2ServerSocket()
{
	if(!GetConfigData())
	{
		::AfxMessageBox("Configuration file not found or corrupt. Please get the correct configuration file Config.txt", MB_OK|MB_ICONSTOP);
		return FALSE;
	}

	int ret = m_pSocket->Connect(m_strIPAddress, m_nPortNum);
	if(ret == 0)
	{
		return FALSE;
	}

	m_bServerOnline = TRUE;

	TRY
	{
	// Construct file object
	m_pSocketFile = new CSocketFile(m_pSocket);
	
	// Construct archive objects for i-o
	m_pArIn = new CArchive(m_pSocketFile, CArchive::load);
	m_pArOut = new CArchive(m_pSocketFile, CArchive::store);
	}
	CATCH(CException, e)
	{
		::AfxMessageBox("Caught!!");
	}
	END_CATCH

	// Construct a message packet object
	m_pMsgPacket = new CMessagePacket();

	return TRUE;
}



/**************************************************************************/
// Method Name	: SendMessagePacket()
// Description	: Sends 1 message packet to the server
// Inputs		: Message packet
// Output		: Sends message packet thru output archive
// Returns		: TRUE if successful, otherwise FALSE
/**************************************************************************/
bool CCommonDialogElements::SendMessagePacket(CMessagePacket& msgPacket)
{
	TRY
	{
		msgPacket.Serialize(*m_pArOut);
		m_pArOut->Flush();

		Sleep(100);
	}
	CATCH(CFileException, e)
	{
		return FALSE;
	}
	END_CATCH

	return TRUE;
}



/**************************************************************************/
// Method Name	: ReceiveMessagePacket()
// Description	: Receives 1 message packet sent by the server
// Inputs		: None
// Output		: Removes all strings from message list inside a message
//				  Receives message packet thru input archive
// Returns		: TRUE if successful, otherwise FALSE
/**************************************************************************/
bool CCommonDialogElements::ReceiveMessagePacket()
{
	// Empty the message list
	m_pMsgPacket->m_msgList.RemoveAll();

	TRY
	{
		m_pMsgPacket->Serialize(*m_pArIn);
		Sleep(100);
	}
	CATCH(CFileException, e)
	{
		return FALSE;
	}
	END_CATCH

	return TRUE;
}



/**************************************************************************/
// Method Name	: GetConfigData()
// Description	: Extracts config data from config file
// Inputs		: None
// Output		: Opens config file called Config.txt
// Returns		: true if successful, otherwise false
/**************************************************************************/
bool CCommonDialogElements::GetConfigData()
{	
	ifstream sourceHandle;			// Handle to config file

	// Open the GLOW log. Don't create it if it wasn't there.
	sourceHandle.open("config.txt", ios::nocreate);

	if(!sourceHandle)
		return false;

	char line[CONFIG_FILE_LINE_LENGTH];
	CString strTemp;	// Used to do string ops on line

	// Copy glow log file to our log file
	while(1)
	{
		// Get a single line from the file
		sourceHandle.getline(line, CONFIG_FILE_LINE_LENGTH);

		// If we have reached the end of file, this last line is useless,
		// so ignore it
		if(sourceHandle.eof())
			break;

		// If this is a comment line, ignore it
		if((line[0] == line[1]) && line[0] == '/')
			continue;

		// Break the line and fill up the config data
		char *key = strtok(line, "=");
		char *value = &line[strlen(key) + 1];

		if(!strcmp(key, "PORT_NUM"))
			m_nPortNum = atoi(value);
		else if(!strcmp(key, "IP_ADDRESS"))
			m_strIPAddress = value;
	}

	// Close the files
	sourceHandle.close();

	return true;
}
