// DeleteFriendsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "MessengerClient.h"
#include "DeleteFriendsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "MessengerClientDoc.h"
#include "MessagePacket.h"


/////////////////////////////////////////////////////////////////////////////
// CDeleteFriendsDlg dialog


CDeleteFriendsDlg::CDeleteFriendsDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CDeleteFriendsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDeleteFriendsDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CDeleteFriendsDlg::CDeleteFriendsDlg(CMessengerClientDoc *pDoc, CWnd* pParent /*=NULL*/)
	: CDialog(CDeleteFriendsDlg::IDD, pParent)
{
	m_pParent = pDoc;
	m_pMsg = new CMessagePacket();
}

void CDeleteFriendsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDeleteFriendsDlg)
	DDX_Control(pDX, IDC_DEL_FRIENDS_LIST, m_lstDelFriends);
	DDX_Control(pDX, IDC_FRIENDS_LIST, m_lstFriends);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDeleteFriendsDlg, CDialog)
	//{{AFX_MSG_MAP(CDeleteFriendsDlg)
	ON_BN_CLICKED(IDC_RIGHT, OnRight)
	ON_BN_CLICKED(IDC_LEFT, OnLeft)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDeleteFriendsDlg message handlers

BOOL CDeleteFriendsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Copy the friend's list here
	
	// Get the view pointer
	CMessengerClientView *pView = m_pParent->GetViewPtr();

	// Copy the friend's list from main view into this dialog
	CopyFriendsTreeToList(pView->m_FriendsList, pView->m_FriendsList.GetRootItem());

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}



/**************************************************************************/
// Method Name	: CopyFriendsTreeToList
// Description	: Copies all friend's names from friend's list (on main
//				  window) into a list box in this dialog
//				  This is a recursive function
// Inputs		: Original friend's list from main view, root of the tree
// Output		: Traverses the tree recursively
//				  If it reaches a leaf node, it checks whether that node is
//				  a friend's name or a group heading
//				  If it's a friend's name, copies it to the list
// Returns		: void
/**************************************************************************/
void CDeleteFriendsDlg::CopyFriendsTreeToList(CTreeCtrl& m_OriginalFriendsList, HTREEITEM root)
{
	// Get the text of the root item
	CString str = m_OriginalFriendsList.GetItemText(root);
	bool bDone = false;			// Return if this var is true

	// Traverse in the tree until you find the item or end of tree
	while(1)
	{
		// If we are to the end of tree
		if(!root || bDone)
			return;

		// If this is a group item, traverse its child trees
		int nImage = 0, nSelectedImage = 0;
		if(m_OriginalFriendsList.GetItemImage(root, nImage, nSelectedImage))
		{
			if((nImage != ONLINE_FRIEND_IMAGE_INDEX) && (nImage != OFFLINE_FRIEND_IMAGE_INDEX))
			{
				// If this item has children, traverse the tree below this item
				if(m_OriginalFriendsList.ItemHasChildren(root))
				{
					HTREEITEM childItem = m_OriginalFriendsList.GetChildItem(root);
					if(childItem)
					{
						CopyFriendsTreeToList(m_OriginalFriendsList, childItem);
					}
					else
					{
						root = m_OriginalFriendsList.GetNextItem(root, TVGN_NEXTVISIBLE);
						CopyFriendsTreeToList(m_OriginalFriendsList, root);
					}
					bDone = true;
				}

				// Otherwise get the next item in the tree
				else
				{
					root = m_OriginalFriendsList.GetNextItem(root, TVGN_NEXTVISIBLE);
					CopyFriendsTreeToList(m_OriginalFriendsList, root);
					return;
				}
			}
			else		// If this is a leaf node
			{
				// Now that we have reached the leaf node, copy the item text into the 
				// Friends list box and go to the next item in the tree
				if(str != "")
					m_lstFriends.InsertItem(m_lstFriends.GetItemCount(), m_OriginalFriendsList.GetItemText(root));
				bDone = true;
				root = m_OriginalFriendsList.GetNextItem(root, TVGN_NEXTVISIBLE);
				CopyFriendsTreeToList(m_OriginalFriendsList, root);
			}
		}
	}
}




/**************************************************************************/
// Method Name	: OnRight
// Description	: Called when user clicks the '>>' button
// Inputs		: None
// Output		: Transfers selected names from left list (all friends)
//				  to right list (selected for deletion)
// Returns		: void
/**************************************************************************/
void CDeleteFriendsDlg::OnRight() 
{
	// Transfer the selected names from left to right
	int nTransferred = TransferNames(m_lstFriends, m_lstDelFriends);

	// If no members on left side are selected
	if(!nTransferred)
	{
		::AfxMessageBox("Please select at least 1 member from left");
		return;
	}

	// Enable the Delete button
	GetDlgItem(IDOK)->EnableWindow();
}



/**************************************************************************/
// Method Name	: TransferNames
// Description	: Called from OnLeft and OnRight methods to transfer
//				  names from one list to another
// Inputs		: source and destination lists
// Output		: Transfers selected names from source list to destination list 
//				  and deletes them from source list
// Returns		: 0 - if no names are selected in source list
//				  Otherwise number of names transferred.
/**************************************************************************/
int CDeleteFriendsDlg::TransferNames(CListCtrl& lstFrom, CListCtrl& lstTo)
{
	// Check whether any items are selected. If not, no need to go further
	int selItemsCount = lstFrom.GetSelectedCount();
	if(!selItemsCount)
		return 0;

	// If at least one item is selected, get the first selected item
	POSITION pos = lstFrom.GetFirstSelectedItemPosition();
	int selItemIndex = lstFrom.GetNextItem(-1, LVNI_SELECTED);

	// This array will store the indexes of items to be removed from list
	int *selItemsIndexesArr = new int[selItemsCount];
	int selItemsIndexesArrIndex = 0;

	while(1)
	{
		// Get the friend's id
		CString selFriend = lstFrom.GetItemText(selItemIndex, 0);

		// Transfer this id to right list box
		lstTo.InsertItem(lstTo.GetItemCount(), selFriend);

		// Put this transferred item's index into the array
		selItemsIndexesArr[selItemsIndexesArrIndex] = selItemIndex;
		selItemsIndexesArrIndex ++;

		// Get the next selected item position. If no item is selected
		// after this, break
		lstFrom.GetNextSelectedItem(pos);
		if(!pos)
			break;

		// Get the text of selected item. This will be friend's id
		selItemIndex = lstFrom.GetNextItem(selItemIndex, LVNI_SELECTED);
	}

	// Delete all items pointed to by the array
	for(int index = selItemsIndexesArrIndex - 1; index >= 0; index--)
		lstFrom.DeleteItem(selItemsIndexesArr[index]);

	// Free the memory allocated to indexes array
	delete[] selItemsIndexesArr;

	return selItemsCount;
}




/**************************************************************************/
// Method Name	: OnLeft
// Description	: Called when user clicks the '<<' button
// Inputs		: None
// Output		: Transfers selected names from right list (selected for deletion)
//				  to right list (rest of the friends)
// Returns		: void
/**************************************************************************/
void CDeleteFriendsDlg::OnLeft() 
{
	// Transfer the selected names from right to left
	int nTransferred = TransferNames(m_lstDelFriends, m_lstFriends);

	// If no members on left side are selected
	if(!nTransferred)
	{
		::AfxMessageBox("Please select at least 1 member from right");
		return;
	}

	// Disable the Delete button, if no items left on the right side
	if(m_lstDelFriends.GetItemCount() == 0)
		GetDlgItem(IDOK)->EnableWindow(FALSE);
}



/**************************************************************************/
// Method Name	: OnOK
// Description	: Called when user clicks 'Delete' button
// Inputs		: None
// Output		: Warns the user abt deletion of names
//				  If user clicks Yes, sends the message to delete these
//				  names to the server 
// Returns		: 0 - if no names are selected in source list
//				  Otherwise number of names transferred.
/**************************************************************************/
void CDeleteFriendsDlg::OnOK() 
{
	// Warn the user
	CString warning = "You are about to delete some friends from your friends list. This operation is not reversible. Are you sure you want to do this?";
	int response = ::AfxMessageBox(warning, MB_YESNO);

	// If user response is negative, abort
	if(response == IDNO)
		CDialog::OnOK();

	// Create a message packet
	CMessagePacket msg;
	msg.m_strFrom	= m_pParent->m_UserName;
	msg.m_strReasonFlag = "DF";					// For Delete Friends

	// Add all the names to the message list inside message packet
	int itemCount = m_lstDelFriends.GetItemCount();
	for(int i = 0; i < itemCount; i++)
	{
		// Get the text for each item
		CString item = m_lstDelFriends.GetItemText(i, 0);
		msg.m_msgList.AddTail(item);
	}

	// Send the message packet
	m_pParent->SendMessagePacket(msg);
	
	CDialog::OnOK();
}
