// ListPane.cpp : implementation file
//

#include "stdafx.h"
#include "MessengerClient.h"
#include "ListPane.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CListPane

IMPLEMENT_DYNCREATE(CListPane, CFormView)

CListPane::CListPane()
	: CFormView(CListPane::IDD)
{
	//{{AFX_DATA_INIT(CListPane)
	//}}AFX_DATA_INIT
	m_bFirstTime = true;
}

CListPane::~CListPane()
{
}

void CListPane::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CListPane)
	DDX_Control(pDX, IDC_MESSAGES_LIST, m_lstMsgs);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CListPane, CFormView)
	//{{AFX_MSG_MAP(CListPane)
	ON_NOTIFY(NM_CLICK, IDC_MESSAGES_LIST, OnClickMessagesList)
	ON_NOTIFY(NM_DBLCLK, IDC_MESSAGES_LIST, OnDblclkMessagesList)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CListPane diagnostics

#ifdef _DEBUG
void CListPane::AssertValid() const
{
	CFormView::AssertValid();
}

void CListPane::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CListPane message handlers


void CListPane::OnInitialUpdate() 
{
	CFormView::OnInitialUpdate();
	
	// Create column headings
	m_lstMsgs.InsertColumn(0, "From", LVCFMT_LEFT, OFFLINE_MESSAGES_DIALOG_COL_LENGTH);
	m_lstMsgs.InsertColumn(1, "Message", LVCFMT_LEFT, OFFLINE_MESSAGES_DIALOG_COL_LENGTH);
	m_lstMsgs.InsertColumn(2, "Received", LVCFMT_LEFT, OFFLINE_MESSAGES_DIALOG_COL_LENGTH);
}



/**************************************************************************/
// Method Name	: GetViewPtr()
// Description	: This method is for getting the view object's pointer
// Inputs		: none
// Output		: Get the View object's pointer. This is same as GetViewPtr()
//				  method is Doc class. So change it when you change the doc
//				  class method, and vice versa
// Returns		: View class pointer (CMessengerClientView *)
/**************************************************************************/
CMessengerClientView * CListPane::GetViewPtr()
{
	CMainFrame *wnd = (CMainFrame *)::AfxGetMainWnd();
	CMDIChildWnd *child = (CMDIChildWnd *)wnd->GetActiveFrame();

	CMessengerClientView *pView = (CMessengerClientView *)child->GetActiveView();

	return pView;
}



/**************************************************************************/
// Method Name	: CallDoubleClickMessagesList()
// Description	: Simply calls OnDblclkMessagesList. Interface for other pane.
// Inputs		: None
// Output		: Get the first selected message
//				  Create a speak window which is ready to send messages
//				  to the sender of that offline message
// Returns		: void
/**************************************************************************/
void CListPane::CallDoubleClickMessagesList()
{
	NMHDR* pNMHDR = NULL;
	LRESULT result = NULL;

	OnDblclkMessagesList(pNMHDR, &result);
}


/**************************************************************************/
// Method Name	: UpdateTextButtonsPane()
// Description	: Used to update the other pane when user scroll thru
//				  this pane.
// Inputs		: None
// Output		: Finds out which message is selected
//				  Puts the text of that message into the text box of other
//				  pane.
// Returns		: void
/**************************************************************************/
void CListPane::UpdateTextButtonsPane()
{
	// See if any of the list members is selected
	POSITION pos = m_lstMsgs.GetFirstSelectedItemPosition();
	if(!pos)
	{
		// Disable the Reply and Delete Buttons
		m_pTextButtonsPane->m_btnDelete.EnableWindow(FALSE);
		m_pTextButtonsPane->m_btnReply.EnableWindow(FALSE);
		return;
	}

	// Get the message body text from the selected position
	int selItemIndex = m_lstMsgs.GetNextItem(-1, LVNI_SELECTED);
	CString messageBody = m_lstMsgs.GetItemText(selItemIndex, 1);

	// Put this message in the Text and Buttons Pane, 
	// Enable Delete and Reply buttons
	m_pTextButtonsPane->m_strMessageBody = messageBody;
	m_pTextButtonsPane->m_btnDelete.EnableWindow();
	m_pTextButtonsPane->m_btnReply.EnableWindow();
	m_pTextButtonsPane->UpdateData(FALSE);
}



/**************************************************************************/
// Method Name	: AdjustFormControls
// Description	: Method to put all controls on the form in their proper places
//				  Called from various places
// Inputs		: void
// Output		: All controls on the form resized and adjusted
// Returns		: void
/**************************************************************************/
void CListPane::AdjustFormControls()
{
	// These rects will contain the coordinates of this pane
	// and that of various controls
	CRect rectWin, rectTextBox;

	// Get main window's rects
	GetWindowRect(&rectWin);

	// Adjust width and height of message text box to half of total size
	rectTextBox = rectWin;

	ScreenToClient(&rectTextBox);
	m_lstMsgs.MoveWindow(rectTextBox.left, rectTextBox.top,
									rectTextBox.Width(), rectTextBox.Height());

	m_bFirstTime = false;
}



/**************************************************************************/
// Method Name	: OnClickMessagesList()
// Description	: When user single clicks any message in this pane, the
//				  message in the other pane is updated accordingly
// Inputs		: system defined
// Output		: Calls the UpdateTextButtonsPane() method.
// Returns		: void
/**************************************************************************/
void CListPane::OnClickMessagesList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	if(!m_lstMsgs.GetItemCount())
		return;

	UpdateTextButtonsPane();

	*pResult = 0;
}



/**************************************************************************/
// Method Name	: OnDblclkMessagesList()
// Description	: When user double clicks any message in this pane, a
//				  speak window is created for the sender
// Inputs		: system defined
// Output		: Get the first selected message
//				  Create a speak window which is ready to send messages
//				  to the sender of that offline message
// Returns		: void
/**************************************************************************/
void CListPane::OnDblclkMessagesList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// See if any of the list members is selected
	POSITION pos = m_lstMsgs.GetFirstSelectedItemPosition();
	if(!pos)
		return;

	// Get the message body and friend's name from the selected position
	int selItemIndex = m_lstMsgs.GetNextItem(-1, LVNI_SELECTED);
	CString fromFriend = m_lstMsgs.GetItemText(selItemIndex, 0);
	CString messageBody = m_lstMsgs.GetItemText(selItemIndex, 1);

	// Get the pointer to the application's view class
	CMessengerClientView *view = GetViewPtr();

	CString str = "";		// TEMP
	view->CreateASpeakWindow(fromFriend, messageBody.GetBuffer(0), str, &view->m_pDoc->m_Preferences.m_Font, view->m_pDoc->m_Preferences.m_iFontColor, view->m_pDoc->m_Preferences.m_iFontSize);

	// Set the flag
	view->m_pDoc->m_pSpeakDlg->m_bNewSpeakWindow = false;

	*pResult = 0;
}



/**************************************************************************/
// Method Name	: OnSize
// Description	: Called when window resizes
// Inputs		: New window coordinates, system defined
// Output		: Calls the base class method
//				  If it's the first time window is being resized, calls method
//				  to resize and adjust controls
// Returns		: void
/**************************************************************************/
void CListPane::OnSize(UINT nType, int cx, int cy) 
{
	CFormView::OnSize(nType, cx, cy);
	
	// Do this only if the window is resized after display
	if(!m_bFirstTime)
		// Resize controls appropriately and place them on proper positions
		AdjustFormControls();
	else
		m_bFirstTime = false;
}
