// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "MessengerClient.h"

#include "MainFrm.h"
#include "ChildFrm.h"
#include "Splash.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_FILE_CHANGE_USER, OnFileChangeUser)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// TODO: add member initialization code here
	
}

CMainFrame::~CMainFrame()
{
}



/**************************************************************************/
// Method Name	: OnCreate
// Description	: Called when creating view
// Inputs		: System defined
// Output		: Creates toolbar & status bar
//				  Calls base class method
// Returns		: 0 - Success
//				  -1 - Failure
/**************************************************************************/
int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		//!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
		!m_wndToolBar.LoadToolBar(IDR_MESSENTYPE))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	// CG: The following line was added by the Splash Screen component.
	CSplashWnd::ShowSplashScreen(this);
	return 0;
}



/**************************************************************************/
// Method Name	: PreCreateWindow
// Description	: Called before showing the window
// Inputs		: System defined
// Output		: Adjusts the width and height of the main window
// Returns		: TRUE - Success
//				  FALSE - Failure
/**************************************************************************/
BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CMDIFrameWnd::PreCreateWindow(cs) )
		return FALSE;

	// Modify the Window width and height
	cs.cx = MAIN_WINDOW_WIDTH;
	cs.cy = MAIN_WINDOW_HEIGHT;

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers



/**************************************************************************/
// Method Name	: OnFileChangeUser
// Description	: Called when user clicks Change User menu item or toolbar button
// Inputs		: None
// Output		: Shows the login dialog
// Returns		: void
/**************************************************************************/
void CMainFrame::OnFileChangeUser() 
{
	ShowUserLoginDialog();
}



/**************************************************************************/
// Method Name	: ShowUserLoginDialog
// Description	: Method to create and show login dialog
// Inputs		: None
// Output		: Obtains view pointer
//				  Checks if conncted to server, if not shows error message
//				  If connected, creates a login dialog and shows it in modal
//				  fashion.
// Returns		: void
/**************************************************************************/
void CMainFrame::ShowUserLoginDialog()
{
	// Get the pointer to the child frame first
	CMDIChildWnd *child = (CMDIChildWnd *)GetActiveFrame();

	CView *pView = child->GetActiveView();
	CMessengerClientDoc *pDoc = (CMessengerClientDoc *)pView->GetDocument();

	
	// If not connected
	if(!pDoc->m_bConnectedFlag)
	{
		AfxMessageBox("Not connected to Server");
		return;
	}

	// Show the messenger Dialog
	pDoc->m_pMessengerLoginDlg = new CMessengerDialog(pDoc, NULL);
	pDoc->m_pMessengerLoginDlg->DoModal();

	pDoc->m_pMessengerLoginDlg = NULL;
}



/**************************************************************************/
// Method Name	: OnCreateClient
// Description	: Called during execution of Create
// Inputs		: Syatem defined
// Output		: Simply calls base class function
// Returns		: TRUE - Success
//				  FALSE - Otherwise
/**************************************************************************/
BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext) 
{
	return CMDIFrameWnd::OnCreateClient(lpcs, pContext);
}