// MessengerClientDoc.cpp : implementation of the CMessengerClientDoc class
//

#include "stdafx.h"
#include "MessengerClient.h"

#include "MessengerClientDoc.h"
#include "OfflineMessagesFrameWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BOOL CMessengerClientDoc::m_bConnectedFlag = FALSE;

/////////////////////////////////////////////////////////////////////////////
// CMessengerClientDoc

IMPLEMENT_DYNCREATE(CMessengerClientDoc, CDocument)

BEGIN_MESSAGE_MAP(CMessengerClientDoc, CDocument)
	//{{AFX_MSG_MAP(CMessengerClientDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(CMessengerClientDoc, CDocument)
	//{{AFX_DISPATCH_MAP(CMessengerClientDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//      DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// Note: we add support for IID_IMessengerClient to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {30B8B544-B68A-11D6-8185-0001031200E7}
static const IID IID_IMessengerClient =
{ 0x30b8b544, 0xb68a, 0x11d6, { 0x81, 0x85, 0x0, 0x1, 0x3, 0x12, 0x0, 0xe7 } };

BEGIN_INTERFACE_MAP(CMessengerClientDoc, CDocument)
	INTERFACE_PART(CMessengerClientDoc, IID_IMessengerClient, Dispatch)
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMessengerClientDoc construction/destruction

CMessengerClientDoc::CMessengerClientDoc()
{
	m_bLoggedInFlag = FALSE;
	m_nNumOnline	= m_nNumOffline = 0;
	m_bServerOnline = FALSE;		// Indicates server is not connected
	m_pOfflineMessagesDlg = NULL;
	m_DeleteFriendsDlg	= NULL;
	m_pPreferencesDlg	= NULL;
	m_bIsBlockedUsersListObtained = false;
	m_pBlockedUsersDlg = NULL;

	AfxOleLockApp();
}

CMessengerClientDoc::~CMessengerClientDoc()
{
	AfxOleUnlockApp();
}



/**************************************************************************/
// Method Name	: OnNewDocument
// Description	: System defined method, called when new doc is created
// Inputs		: None
// Output		: Calls base class method
//				  Sets the title to 'Not Logged In'
//				  Reads application settings from registry and stores them in preferences
//				  Checks whether we are already connected to server, if not, connects
// Returns		: TRUE - Success
//				  FALSE - Otherwise
/**************************************************************************/
BOOL CMessengerClientDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// Set the doc title to Not Logged in
	ResetTitle();

	// Get app settings first
	GetApplicationSettingsFromRegistry();

	// If already connected, return
	if(m_bConnectedFlag)
		return TRUE;

	// Otherwise connect to the server
	ConnectToServer();

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CMessengerClientDoc serialization

void CMessengerClientDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CMessengerClientDoc diagnostics

#ifdef _DEBUG
void CMessengerClientDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CMessengerClientDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMessengerClientDoc commands



/**************************************************************************/
// Method Name	: ProcessMessagePacket
// Description	: Central control for the app. This process reads an incoming
//				  Message packet, and calls appropriate methods to act on it.
// Inputs		: None
// Output		: Reads the reason flag in the incoming message packet
//				  Depending on this flag, does various actions as follows:
//					Flag Meaning			Action
//					-----------------------------------------------------
//					Z	 Server shutdown	Log off	and disconnect
//					ZX	 Admin action		Log off	and disconnect
//					SL	 Login success		Store username and password
//											Ask for friend's list to server
//					SLU	 Friend's list		Save login preferences to registry
//						 rcvd				Ask for offline messages		
//					SR	 Registration		Closes registration dialog
//						 successful			Shows success message
//					SM	 Message received	Process it using appro method
//					SN	 Someone has come	If he is a friend, put him in
//						 online				online list, remove from offline
//					SX	 Someone has gone	If he is a friend, put him in
//						 offline			offline list, remove from online list
//					SFA	 Someone sent		Create and show 'Accept and Add' dialog
//						 'Add as Friend'	and initialize it
//						 request
//					SFN  Someone accepted	Put that user in online friends list
//						 invitation and is
//						 online
//					SFX  Someone accepted	Put that user in offline friends list
//						 invitation and is
//						 offline
//					SC	 Change password	Store new password
//						 success			Close change password dialog
//					SD	 Friends deletion	Clear friend's list
//						 success			Request friend's list again.
//					OL	 Offline msg rcvd	Show offline msgs window
//											Send acknowledgement to server
//					BU	 Blocked users list	Populate list with values and show
//						 rcvd				in a dialog
//					FL	 Login failed		Make login dialog visible
//					F	 Any other failure	Show failure message
//				  Reads application settings from registry and stores them in preferences
//				  Checks whether we are already connected to server, if not, connects
// Returns		: TRUE - Success
//				  FALSE - Otherwise
/**************************************************************************/
bool CMessengerClientDoc::ProcessMessagePacket()
{
	// Critical message, server is shutting down
	if(m_pMsgPacket->m_strReasonFlag == "Z")
	{
		::AfxMessageBox("Server is shutting down. Logging you off ...");
		m_bServerOnline = FALSE;		// Indicates server is not connected
		DoCleanup();
	}

	// Admin logged this user off
	else if(m_pMsgPacket->m_strReasonFlag == "ZX")
	{
		DoCleanup();

		m_bServerOnline = FALSE;		// Indicates server is not connected
										// Do this to prevent re login
		::AfxMessageBox("Admin has logged you off ...");
	}

	// If request is successful
	else if(m_pMsgPacket->m_strReasonFlag.GetAt(0) == 'S')
	{
		// If login request is successful
		if(m_pMsgPacket->m_strReasonFlag.GetAt(1) == 'L')
		{	
			// If a friends list is received
			if((m_pMsgPacket->m_strReasonFlag.GetLength() > 2) && (m_pMsgPacket->m_strReasonFlag.GetAt(2) == 'U'))
			{
				// Close the login dialog, only if we are doing login
				if(!m_bLoggedInFlag)
				{
					// Write registry entries from the dialog before closing it
					m_Preferences.m_bAutoLogin		= m_pMessengerLoginDlg->m_bAutoLogin;
					m_Preferences.m_bRememberLogin	= m_pMessengerLoginDlg->m_bRememberLogin;
					m_Preferences.m_UserName		= m_pMessengerLoginDlg->m_UserName;
					m_Preferences.m_Password		= m_pMessengerLoginDlg->m_Password;

					// Write these changes to registry
					this->ApplyPreferenceChanges(LOGIN_PREFERENCES);

					m_pMessengerLoginDlg->EndDialog(0);
				}

				// Show this data on screen
				ShowFriendsList(m_pMsgPacket);

				// Now request offline messages, if we are logging in for now
				if(!m_bLoggedInFlag)
				{
					CMessagePacket msg;
					msg.m_strFrom = m_pMsgPacket->m_strTo;
					msg.m_strReasonFlag = "SLO";			
					SendMessagePacket(msg);
				}

				// Now we are fully logged in
				m_bLoggedInFlag = TRUE;

				return TRUE;
			}

			// Store Username here
			m_UserName = m_pMessengerLoginDlg->m_UserName;
			m_Password = m_pMessengerLoginDlg->m_Password;
			
			// Login done, demand friends list
			RequestFriendsList();
		}

		// If registration request is successful
		else if(m_pMsgPacket->m_strReasonFlag.GetAt(1) == 'R')
		{
			// Close the registration dialog
			int n = 0;
			m_pMessengerLoginDlg->m_pRegisterDlg->EndDialog(n);

			::AfxMessageBox("Registration success, proceed to Login");
		}

		// If it's a message packet from somebody
		else if(m_pMsgPacket->m_strReasonFlag.GetAt(1) == 'M')
		{
			ProcessMessageFromOtherUser(m_pMsgPacket);
		}
		
		// If it's a user online notification
		else if(m_pMsgPacket->m_strReasonFlag.GetAt(1) == 'N')
		{
			// See if this user is in friends list
			HTREEITEM user = FindFriendFromList(m_pMsgPacket);

			// If he is a friend, then we bother abt adding
			if(user)
			{
				// First remove this friend from list. This will ensure his name is not
				// there in offline friends' list
				RemoveFriendFromList(m_pMsgPacket, user);

				// Now we can safely add him in the online friends' list
				AddFriendToList(m_pMsgPacket, TRUE);
			}
		}
		
		// If a friend has logged off
		else if(m_pMsgPacket->m_strReasonFlag.GetAt(1) == 'X')
		{
			// See if this user is in friends list
			HTREEITEM user = FindFriendFromList(m_pMsgPacket);

			// If he is a friend, then we bother abt adding
			if(user)
			{
				// Remove this friend's name from list. This will ensure his name is not
				// there in online friends' list
				RemoveFriendFromList(m_pMsgPacket, user);

				// Now we can safely add him in the offline friends' list
				AddFriendToList(m_pMsgPacket, FALSE);
			}
		}
		// If it's a request about someone adding someone to friend's list
		else if(m_pMsgPacket->m_strReasonFlag.GetAt(1) == 'F')
		{
			if(m_pMsgPacket->m_strReasonFlag.GetLength() > 2)  
			{
				// If someone else has tried to add this user as his friend
				if(m_pMsgPacket->m_strReasonFlag.GetAt(2) == 'A')
				{
					// Create a dialog to ask the user whether he wants to accept the other 
					// user as his friend
					m_pAcceptAndAddDlg = new CAcceptAndAddDlg(this, NULL);
					m_pAcceptAndAddDlg->Create(IDD_ACCEPT_ADD_DIALOG, GetViewPtr());
					
					// Initialize the vars in dialog
					m_pAcceptAndAddDlg->m_strFriendMsg = m_pMsgPacket->m_strMessageBody;
					m_pAcceptAndAddDlg->m_pMsg->m_strTo = m_pMsgPacket->m_strFrom;
					m_pAcceptAndAddDlg->UpdateData(FALSE);
					m_pAcceptAndAddDlg->SetFocus();
					m_pAcceptAndAddDlg->ShowWindow(SW_SHOW);

					return TRUE;
				}
				// If someone has accepted the invitation of this user to be
				// added to the friend's list and is online
				else if(m_pMsgPacket->m_strReasonFlag.GetAt(2) == 'N')
				{
					AddFriendToList(m_pMsgPacket, TRUE);
					return TRUE;
				}
				// If someone has accepted the invitation of this user to be
				// added to the friend's list and is offline (Unlikely case, but just in case ...)
				else if(m_pMsgPacket->m_strReasonFlag.GetAt(2) == 'X')
				{
					AddFriendToList(m_pMsgPacket, FALSE);
					return TRUE;
				}
			}

			// Just inform this user abt the message being sent
			::AfxMessageBox("A message has been sent to " + m_pMsgPacket->m_strFrom + " asking for his/her permission");
		}
		else if(m_pMsgPacket->m_strReasonFlag.GetAt(1) == 'C')		// Password change success
		{
			// Record the new password in here
			m_Password = m_pChangePasswordDlg->m_Password2;

			// Close the change password dialog
			int x = 0;
			m_pChangePasswordDlg->EndDialog(x);
			m_pChangePasswordDlg = NULL;

			::AfxMessageBox("Password changed successfully");
		}
		else if(m_pMsgPacket->m_strReasonFlag.GetAt(1) == 'D')		// Delete friends success
		{
			::AfxMessageBox("Selected Friends deleted successfully");

			// Clear the existing friends list in the tree and request a new one
			ClearFriendsList();		
			RequestFriendsList();
		}
		else if(m_pMsgPacket->m_strReasonFlag.GetAt(1) == 'B')		// Blocked users removal success
		{
			// Make the dialog visible, show success message, and close the dialog
			m_pBlockedUsersDlg->ShowWindow(SW_SHOW);
			::AfxMessageBox("The selected users are now unblocked");
			m_pBlockedUsersDlg->EndDialog(0);

			// Reset this var, so that next time 'remove blocked users' dialog 
			// is made visible, it's in-sync with server
			m_bIsBlockedUsersListObtained = false;

			// Clear the list at client's end
			EmptyBlockedUsersList();
		}
	}

	// Offline message from another user
	else if(m_pMsgPacket->m_strReasonFlag.GetAt(0) == 'O')
	{
		// If this is the last offline message, ignore the contents,
		// and just show the dialog
		if((m_pMsgPacket->m_strReasonFlag.GetLength() > 1) && (m_pMsgPacket->m_strReasonFlag.GetAt(1) == 'L') && (!m_OfflineMessagesList.IsEmpty()))
		{
			CreateAnOfflineMessagesDialog();
			return TRUE;
		}

		// Write this offline message at the client end
		//WriteOffLineMessageToDatabase(m_pMsgPacket);
		//Sleep(200);

		// First add this message to the offline messages list
		CMessagePacket *pMsg = new CMessagePacket(*m_pMsgPacket);
		m_OfflineMessagesList.AddTail(pMsg);

		// Send acknowledgement to the server
		m_pMsgPacket->m_strFrom = m_UserName;
		m_pMsgPacket->m_strReasonFlag = "ON";
		SendMessagePacket(*m_pMsgPacket);
	}

	// Blocked users list received
	else if(m_pMsgPacket->m_strReasonFlag.GetAt(0) == 'B')
	{
		// Put this list into the memory
		PopulateBlockedUsersList(m_pMsgPacket);

		// Show the blocked users list dialog
		m_pBlockedUsersDlg = new CBlockedUsersListDlg(this);

		m_pBlockedUsersDlg->DoModal();

		// Free memory
		delete(m_pBlockedUsersDlg);
		m_pBlockedUsersDlg = NULL;
	}

	// Any kind of failure
	else if(m_pMsgPacket->m_strReasonFlag.GetAt(0) == 'F')
	{
		AfxMessageBox(m_pMsgPacket->m_strFrom);

		// If login has failed, make the login window visible again
		if((m_pMsgPacket->m_strReasonFlag.GetLength() > 1) && (m_pMsgPacket->m_strReasonFlag.GetAt(1) == 'L'))
			m_pMessengerLoginDlg->ShowWindow(SW_SHOW);
	}

	return TRUE;
}



/**************************************************************************/
// Method Name	: ShowFriendsList
// Description	: Prepares the friend's list in a tree view
// Inputs		: Message packet. The string list in this packet will contain
//				  friend's list. Each online friend's name will be appended
//				  by a '*'.
// Output		: Sets user name as the window title
//				  Gets each friend's name from packet's string list and puts it 
//					into tree view
//				  Shows the number of online and offline friends
//				  Set this window as active window
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::ShowFriendsList(CMessagePacket *m_pMsgPacket)
{
	SetTitle(m_UserName);

	// Show the list
	if(m_pMsgPacket->m_msgList.IsEmpty())
		return;

	CMessengerClientView *pView = GetViewPtr();

	while(1)
	{
		// Take the user name from the friends list
		CString str = m_pMsgPacket->m_msgList.RemoveHead();

		if(str.GetAt(str.GetLength() - 1) == '*')	// The friend is online
		{
			m_nNumOnline ++;
			str.TrimRight('*');

			// Add it to the tree on client dialog
			pView->m_FriendsList.InsertItem(str, ONLINE_FRIEND_IMAGE_INDEX, ONLINE_FRIEND_IMAGE_INDEX, m_hOnline);
		}
		else										// The friend is offline
		{
			m_nNumOffline ++;

			// Add it to the tree on client dialog
			pView->m_FriendsList.InsertItem(str, OFFLINE_FRIEND_IMAGE_INDEX, OFFLINE_FRIEND_IMAGE_INDEX, m_hOffline);
		}

		if(m_pMsgPacket->m_msgList.IsEmpty())
			break;
	}

	// Update the number of online and offline friends
	ShowOnlineOfflineSummary();

	// Get the focus on current messenger window
	pView->SetForegroundWindow();
}



/**************************************************************************/
// Method Name	: AddFriendToList
// Description	: Adds a single name to the friend's tree view
// Inputs		: Message packet. The From field in this packet will contain
//					friend's name.
//				  A flag indicating whether he is online or offline
// Output		: Puts friend's name into the list. It goes into online/offline
//					branch depending on the flag
//				  Shows number of online and offline friends
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::AddFriendToList(CMessagePacket *m_pMsgPacket, BOOL online)
{
	CMessengerClientView *pView = GetViewPtr();

	// Add this name to the friends tree in the client dialog

	// If online, add this name to online friends' list
	if(online)
	{
		pView->m_FriendsList.InsertItem(m_pMsgPacket->m_strFrom, ONLINE_FRIEND_IMAGE_INDEX, ONLINE_FRIEND_IMAGE_INDEX, m_hOnline);
		m_nNumOnline ++;

		if(m_nNumOffline)
			m_nNumOffline --;
	}
	else
	{
		pView->m_FriendsList.InsertItem(m_pMsgPacket->m_strFrom, OFFLINE_FRIEND_IMAGE_INDEX, OFFLINE_FRIEND_IMAGE_INDEX, m_hOffline);
		m_nNumOffline ++;

		if(m_nNumOnline)
			m_nNumOnline --;
	}

	// Update the number of online and offline friends
	ShowOnlineOfflineSummary();
}



/**************************************************************************/
// Method Name	: RemoveFriendFromList
// Description	: Removes a friend's name from the friend's tree view
// Inputs		: Message packet.
//				  Tree item for the friend to be removed
// Output		: The requsted friend is removed from the tree
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::RemoveFriendFromList(CMessagePacket *m_pMsgPacket, HTREEITEM& user)
{
	CMessengerClientView *pView = GetViewPtr();

	// Remove this name from the friends tree in the client dialog, if found
	if(user)
		pView->m_FriendsList.DeleteItem(user);
}



/**************************************************************************/
// Method Name	: FindFriendFromList
// Description	: Finds a friend's name from the friend's tree view
// Inputs		: Message packet.
// Output		: None
// Returns		: Reference to the friend's tree node
/**************************************************************************/
HTREEITEM CMessengerClientDoc::FindFriendFromList(CMessagePacket *m_pMsgPacket)
{
	CMessengerClientView *pView = GetViewPtr();

	// Go to the the root of tree ("Friends List")
	HTREEITEM retItem = pView->m_FriendsList.GetChildItem(pView->m_FriendsList.GetRootItem());

	retItem = TraverseTree(retItem, pView);

	return retItem;
}



/**************************************************************************/
// Method Name	: TraverseTree
// Description	: Finds a friend's name from the friend's tree view
//				  This is a recursive function
// Inputs		: Root of the tree
//				  view pointer
// Output		: None
// Returns		: Reference to the friend's tree node
/**************************************************************************/
HTREEITEM CMessengerClientDoc::TraverseTree(HTREEITEM root, CMessengerClientView *pView)
{
	HTREEITEM retItem;

	// Traverse in the tree until you find the item or end of tree
	while(1)
	{
		// If we are to the end of tree
		if(!root)
			return NULL;

		// If this is a group item, traverse its child trees
		int nImage = 0, nSelectedImage = 0;
		if(pView->m_FriendsList.GetItemImage(root, nImage, nSelectedImage))
		{
			if((nImage != ONLINE_FRIEND_IMAGE_INDEX) && (nImage != OFFLINE_FRIEND_IMAGE_INDEX))
			{
				// If this item has children, traverse the tree below this item
				if(pView->m_FriendsList.ItemHasChildren(root))
				{
					retItem = TraverseTree(pView->m_FriendsList.GetChildItem(root), pView);

					// If found, return it, else go to the next sibling of the parent
					if(retItem)
						return retItem;
				}

				// Otherwise get the next item in the tree
				else
				{
					root = pView->m_FriendsList.GetNextItem(root, TVGN_NEXTVISIBLE);
					return(TraverseTree(root, pView));
				}
			}
		}

		// Now compare the item text with the name in message packet
		if(m_pMsgPacket->m_strFrom == pView->m_FriendsList.GetItemText(root))
			return root;
		else
		{
			root = pView->m_FriendsList.GetNextItem(root, TVGN_NEXTVISIBLE);
			return(TraverseTree(root, pView));
		}
	}
	
	return NULL;
}



/**************************************************************************/
// Method Name	: FindSpeakDialogForFriend
// Description	: Finds whether a speak dialog for a friend is open
//				  and returns its pointer
// Inputs		: Friend's id
// Output		: None
// Returns		: Pointer to the speak dialog of friend if open
//				  NULL otherwise
/**************************************************************************/
CSpeakWindow *CMessengerClientDoc::FindSpeakDialogForFriend(CString strFriend)
{
	// Check if the window list is empty
	if(m_SpeakDialogsList.IsEmpty())
		return NULL;

	// See if a window for friend is open. For this iterate thru the Speak windows list
	CSpeakWindow *speakPtr = (CSpeakWindow *)m_SpeakDialogsList.GetHead();
	CSpeakWindow *tailPtr = (CSpeakWindow *)m_SpeakDialogsList.GetTail();

	// Store the head position
	POSITION pos = m_SpeakDialogsList.GetHeadPosition();

	while(1)
	{
		// If window found
		if(speakPtr->m_strTo == strFriend)
			return speakPtr;

		// If we ve reached end of list
		if(speakPtr == tailPtr)
			break;

		// Else go ahead 1 position
		speakPtr = (CSpeakWindow *)m_SpeakDialogsList.GetNext(pos);
	}

	return NULL;
}



/**************************************************************************/
// Method Name	: RemoveSpeakDialogFromList
// Description	: Removes a speak dialog from Speak dialogs list
//				  Called when a speak dialog closes
// Inputs		: Friend's id
// Output		: Speak dialog pointer removed from list
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::RemoveSpeakDialogFromList(CString strFriend)
{
	// Check if the window list is empty
	if(m_SpeakDialogsList.IsEmpty())
		return;

	// See if a window for friend is open. For this iterate thru the Speak windows list
	CSpeakWindow *speakPtr = (CSpeakWindow *)m_SpeakDialogsList.GetHead();
	CSpeakWindow *tailPtr = (CSpeakWindow *)m_SpeakDialogsList.GetTail();

	// Store the head position
	POSITION pos = m_SpeakDialogsList.GetHeadPosition();
	speakPtr = (CSpeakWindow *)m_SpeakDialogsList.GetNext(pos);

	while(1)
	{
		// If window found
		if(speakPtr->m_strTo == strFriend)
		{
			pos = m_SpeakDialogsList.Find(speakPtr);
			m_SpeakDialogsList.RemoveAt(pos);
			break;
		}

		// If we ve reached end of list
		if(speakPtr == tailPtr)
			break;

		// Else go ahead 1 position
		speakPtr = (CSpeakWindow *)m_SpeakDialogsList.GetNext(pos);
	}
}



/**************************************************************************/
// Method Name	: GetViewPtr
// Description	: Finds view pointer. Used by many methods in this class
// Inputs		: None
// Output		: None
// Returns		: view pointer
/**************************************************************************/
CMessengerClientView *CMessengerClientDoc::GetViewPtr()
{
	CMainFrame *wnd = (CMainFrame *)::AfxGetMainWnd();
	CMDIChildWnd *child = (CMDIChildWnd *)wnd->GetActiveFrame();

	CMessengerClientView *pView = (CMessengerClientView *)child->GetActiveView();

	return pView;
}



/**************************************************************************/
// Method Name	: ShowOnlineOfflineSummary
// Description	: Shows online and offline friend's number in friends tree view
// Inputs		: None
// Output		: Changes Online Friends and Offline Friends braches to append
//				  number of leaf nodes under them.
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::ShowOnlineOfflineSummary()
{
	// Get num of friends online and offline in char vars
	char charOnline[5];
	itoa(m_nNumOnline, charOnline, 10);
	char charOffline[5];
	itoa(m_nNumOffline, charOffline, 10);

	// Get the view pointer
	CMessengerClientView *pView = GetViewPtr();
	CString item = "Online Friends (";
	item += charOnline;
	item += ")"; 

	// Show online friends note
	pView->m_FriendsList.SetItem(m_hOnline, TVIF_CHILDREN|TVIF_TEXT|TVIF_STATE, item, ONLINE_FRIENDS_LIST_IMAGE_INDEX, ONLINE_FRIENDS_LIST_IMAGE_INDEX, TVIS_BOLD|TVIS_EXPANDED, TVIS_BOLD|TVIS_EXPANDED, NULL);

	item = "Offline Friends (";
	item += charOffline;
	item += ")"; 
	pView->m_FriendsList.SetItem(m_hOffline, TVIF_CHILDREN|TVIF_TEXT|TVIF_STATE, item, OFFLINE_FRIENDS_LIST_IMAGE_INDEX, OFFLINE_FRIENDS_LIST_IMAGE_INDEX, TVIS_EXPANDED, TVIS_EXPANDED, NULL);
}



/**************************************************************************/
// Method Name	: CreateAnOfflineMessagesDialog
// Description	: Creates offline messages window, if not existing
//				  Puts messages into it otherwise.
// Inputs		: None
// Output		: If offline messages window doesn't exist, creates a frame
//					and initializes the form view elements
//				  Inserts messages into the offline messages window
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::CreateAnOfflineMessagesDialog()
{
	COfflineMessagesFrameWnd *pFrame;

	// If offline messages dialog is visible, just return
	if(!m_pOfflineMessagesDlg)
	{
		// Get the document template
		CMultiDocTemplate *pDocTemplate = ((CMessengerClientApp *)::AfxGetApp())->pOfflineMessagesDocTemplate;

		// Using this doc template, create a new frame window
		pFrame = (COfflineMessagesFrameWnd *)((CMessengerClientApp *)::AfxGetApp())->pOfflineMessagesDocTemplate->CreateNewFrame(this, NULL);

		if(!pFrame)
		{
			::AfxMessageBox("Could not create offline messages dialog due to internal error");
			return;
		}

		// Put the frame onto predetermined coordinates and size
		pFrame->MoveWindow(OFFLINE_MESSAGES_DIALOG_X_POS, OFFLINE_MESSAGES_DIALOG_Y_POS
						   , OFFLINE_MESSAGES_DIALOG_WIDTH, OFFLINE_MESSAGES_DIALOG_HEIGHT);

		// Refresh the frame
		pFrame->InitialUpdateFrame(this, true);

		// Set titles for frame as well as current view
		pFrame->SetWindowText("Offline Messages -- " + m_UserName);

		// Store the offline messages view pointer
		//m_pOfflineMessagesDlg = (COfflineMessagesDlg *)pFrame->GetActiveView();
		m_pOfflineMessagesDlg = (COfflineMessagesWindow *)pFrame->GetActiveView();

		// Store the Frame's address in the view class
		m_pOfflineMessagesDlg->m_pFrame = pFrame;
	}

	m_pOfflineMessagesDlg->SetFocus();
	m_pOfflineMessagesDlg->ShowWindow(SW_SHOW);

	// Else insert the message into the offline messages dialog
	InsertMessagesIntoOfflineMessagesDialog(pFrame);
}



/**************************************************************************/
// Method Name	: InsertMessagesIntoOfflineMessagesDialog
// Description	: Inserts messages into the offline messages window
// Inputs		: Pointer to frame window
// Output		: If there are any offline messages, puts them one by one into
//					the list pane of offline messages window
//				  When all the messages are put in, selects the first message
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::InsertMessagesIntoOfflineMessagesDialog(COfflineMessagesFrameWnd *pFrame)
{
	// If the offline messages list is empty, no need to go further
	if(m_OfflineMessagesList.IsEmpty())
		return;

	int numOfMessages = 0;

	// Get the address of the list pane
	//COfflineMessagesListPane *pListPane = (COfflineMessagesListPane *)(pFrame->m_wndSplitter.GetPane(0, 0));
	CListPane *pListPane = (CListPane *)(pFrame->m_wndSplitter.GetPane(0, 0));

	// Get the address of the text pane
	CTextPane *pTextButtonsPane = (CTextPane *)(pFrame->m_wndSplitter.GetPane(1, 0));

	// Put each other's refernces in both the list panes
	pListPane->m_pTextButtonsPane = pTextButtonsPane;
	pTextButtonsPane->m_pMsgsListPane = pListPane;

	// These vars are used to traverse thru the message list
	CMessagePacket *msgPtr = (CMessagePacket *)m_OfflineMessagesList.GetHead();
	CMessagePacket *tailPtr = (CMessagePacket *)m_OfflineMessagesList.GetTail();
	CMessagePacket *nextPtr = NULL;

	POSITION pos = m_OfflineMessagesList.Find(msgPtr);

	msgPtr = (CMessagePacket *)m_OfflineMessagesList.GetNext(pos);

	while(1)
	{
		// Get the number of items in the message list
		numOfMessages  = pListPane->m_lstMsgs.GetItemCount();

		// Set the data in the dialog
		pListPane->m_lstMsgs.InsertItem(numOfMessages, msgPtr->m_strFrom);
		pListPane->m_lstMsgs.SetItemText(numOfMessages, 1, msgPtr->m_strMessageBody);
		pListPane->m_lstMsgs.SetItemText(numOfMessages, 2, msgPtr->m_MessageTime.Format("%A, %B, %d, %Y, %H:%M:%S"));

		// If this is not the last entry in the list, record the next position
		if(msgPtr != tailPtr)
			nextPtr = (CMessagePacket *)m_OfflineMessagesList.GetNext(pos);

		// If we are at the end of list, get out
		else
			break;

		// If we have not reached end of list, go one position ahead
		msgPtr = nextPtr;
	}

	pListPane->UpdateData(FALSE);
	pListPane->SetFocus();

	// Select the first message
	LVITEM	item;
	item.mask = LVIF_STATE|LVIF_DI_SETITEM;
	item.iItem = item.iSubItem = 0;
	item.state = LVIS_SELECTED|LVIS_FOCUSED;
	item.stateMask = LVIS_SELECTED|LVIS_FOCUSED;
	
	pListPane->m_lstMsgs.SetItemState(0, &item);
	pListPane->UpdateTextButtonsPane();
}



/**************************************************************************/
// Method Name	: WriteOffLineMessageToDatabase
// Description	: Writes messages from the offline messages window into database
//				  NOT BEING USED YET
// Inputs		: Message packet
// Output		: 
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::WriteOffLineMessageToDatabase(CMessagePacket *pMsgPacket)
{
	// Create a file handle if not already open
	if(!m_fOfflineMessagesFile.is_open())
		m_fOfflineMessagesFile.open("Offline.txt", ios::out | ios::app );

	// Write the message to file
	m_fOfflineMessagesFile.write(pMsgPacket->m_strFrom, pMsgPacket->m_strFrom.GetLength());
	m_fOfflineMessagesFile.write(pMsgPacket->m_strMessageBody, pMsgPacket->m_strMessageBody.GetLength());
	m_fOfflineMessagesFile.write(pMsgPacket->m_MessageTime.Format("%A, %B, %d, %Y, %H:%M:%S"), 20);

	// Close the file
	m_fOfflineMessagesFile.close();
}



/**************************************************************************/
// Method Name	: ConnectToServer
// Description	: Connects to server
// Inputs		: None
// Output		: Creates a socket and connects to server
// Returns		: void
/**************************************************************************/
BOOL CMessengerClientDoc::ConnectToServer()
{
	// Create a client socket
	m_pSocket = new CMessengerClientSocket(this);

	if(!m_pSocket->Create())
	{
		AfxMessageBox("Could not create socket");
		return FALSE;
	}

	// Try connecting to the Server
	if(!Connect2ServerSocket())
	{
		//AfxMessageBox("Connection failed");
		return FALSE;
	}

	// Set the connected flag to TRUE. This way everyone in the system will come
	// to know that the connection is active
	m_bConnectedFlag = TRUE;

	return TRUE;
}



/**************************************************************************/
// Method Name	: DisconnectFromServer
// Description	: Disconnects from server
// Inputs		: None
// Output		: Deletes the socket
//				  Clears friend's list from view
//				  Resets the window title to 'Not logged in'
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::DisconnectFromServer()
{
	// Delete the socket
	if(m_pSocket)
	{
		delete(m_pSocket);
		m_pSocket = NULL;
	}

	// Clear the friend's list
	ClearFriendsList();

	m_bConnectedFlag = FALSE;

	// Set the doc title to Not Logged in
	ResetTitle();
}


/**************************************************************************/
// Method Name	: SaveModified
// Description	: Called by system when doc is closed
// Inputs		: None
// Output		: If server is online and we are connected
//					Send message to the server abt our exit
//					Wait for some time. It makes sure message is not lost.
//					Delete the socket.
//				  Clear all lists, free memory
//				  Reset the window title to 'Not logged in'
//				  Call base class method
// Returns		: void
/**************************************************************************/
BOOL CMessengerClientDoc::SaveModified() 
{
	// Find out which doc closed. We want to break the connection only if it's
	// the main document that's closing
	CDocTemplate *pDocTemplate = GetDocTemplate();

	// Notify the server that this user has exited, if server is running
	// and we are connected to it
	CMessagePacket msg;

	if(m_bServerOnline && m_bConnectedFlag)
	{
		msg.m_strFrom = m_UserName;
		msg.m_strTo = m_UserName;
		msg.m_strReasonFlag = "X";

		SendMessagePacket(msg);

		// Wait for some time to make sure the message went on the socket
		Sleep(200);
		
		// Destroy the created socket. The destructor will auto close the socket
		if(m_pSocket)
		{
			delete(m_pSocket);
			m_pSocket = NULL;
		}
	}
	
	// Do this so that now there's no connection with server
	m_bLoggedInFlag = FALSE;
	m_bConnectedFlag = FALSE;

	// Clear all the lists
	EmptyAllLists();

	// Reset the doc title
	ResetTitle();

	return CDocument::SaveModified();
}



/**************************************************************************/
// Method Name	: DoCleanup
// Description	: Called by other objects to call SaveModified
//				  because SaveModified is Protected.
// Inputs		: None
// Output		: None
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::DoCleanup()
{
	SaveModified();
}



/**************************************************************************/
// Method Name	: EmptyAllLists
// Description	: Calls other methods to clear list objects and free memory
//				  Imp cleanup method
// Inputs		: None
// Output		: None
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::EmptyAllLists()
{
	// Clear the offline messages list
	EmptyOfflineMessagesList();
	
	// If any speak dialogs are open, close them and clear them from memory
	EmptySpeakDialogsList();

	// Clear the blocked users list
	EmptyBlockedUsersList();

	// Clear friend's tree from screen
	ClearFriendsList();
}



/**************************************************************************/
// Method Name	: EmptyOfflineMessagesList
// Description	: Clears offline messages list
// Inputs		: None
// Output		: Frees the memory occupied by message packets, whose addresses
//					are in offline messages list
//				  Empties the list itself.
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::EmptyOfflineMessagesList()
{
	if(m_OfflineMessagesList.IsEmpty())
		return;

	// Empty the socket clients list
	CMessagePacket *msgPtr = (CMessagePacket *)m_OfflineMessagesList.GetHead();
	CMessagePacket *tailPtr = (CMessagePacket *)m_OfflineMessagesList.GetTail();
	CMessagePacket *nextPtr = NULL;

	POSITION pos = m_OfflineMessagesList.Find(msgPtr);

	msgPtr = (CMessagePacket *)m_OfflineMessagesList.GetNext(pos);

	while(1)
	{
		// If this is not the last entry in the list, record the next position
		if(msgPtr != tailPtr)
		{
			nextPtr = (CMessagePacket *)m_OfflineMessagesList.GetNext(pos);
		}

		// Delete the socket pointed by the current position
		delete(msgPtr);

		// If we are at the end of list, get out
		if(msgPtr == tailPtr)
			break;

		// If we have not reached end of list, go one position ahead
		msgPtr = nextPtr;
	}

	// Now the memory pointed to by all the pointers in list is freed
	// empty the list itself
	m_OfflineMessagesList.RemoveAll();
}



/**************************************************************************/
// Method Name	: EmptySpeakDialogsList
// Description	: Clears speak dialogs list
// Inputs		: None
// Output		: Closes all speak dialogs
//				  Empties the speak dialogs list itself
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::EmptySpeakDialogsList()
{
	if(m_SpeakDialogsList.IsEmpty())
		return;

	// Empty the socket clients list
	CSpeakWindow *speakPtr = (CSpeakWindow *)m_SpeakDialogsList.GetHead();
	CSpeakWindow *tailPtr = (CSpeakWindow *)m_SpeakDialogsList.GetTail();
	CSpeakWindow *nextPtr = NULL;

	POSITION pos = m_SpeakDialogsList.Find(speakPtr);

	speakPtr = (CSpeakWindow *)m_SpeakDialogsList.GetNext(pos);

	while(1)
	{
		// If this is not the last entry in the list, record the next position
		if(speakPtr != tailPtr)
		{
			nextPtr = (CSpeakWindow *)m_SpeakDialogsList.GetNext(pos);
		}

		// Delete the speak dialog pointed by the current position
		int x = 0;
		speakPtr->SendMessage(WM_QUIT);

		// If we are at the end of list, get out
		if(speakPtr == tailPtr)
			break;

		// If we have not reached end of list, go one position ahead
		speakPtr = nextPtr;
	}

	// Now the memory pointed to by all the pointers in list is freed
	// empty the list itself
	m_SpeakDialogsList.RemoveAll();
}




/**************************************************************************/
// Method Name	: EmptyBlockedUsersList
// Description	: Clears blocked users list
// Inputs		: None
// Output		: If blocked users list is obtained, deletes each list
//					element from memory
//				  Empties the blocked users list itself
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::EmptyBlockedUsersList()
{
	// If list is not obtained or is empty, no need to go further
	if(m_BlockedUsersList.IsEmpty())
		return;

	// Get addresses of head and tail members of the list
	CMessagePacket *pHeadMsg = (CMessagePacket *)m_BlockedUsersList.GetHead();
	CMessagePacket *pTailMsg = (CMessagePacket *)m_BlockedUsersList.GetTail();
	CMessagePacket *pNextMsg = NULL;

	// Store the head position
	POSITION pos = m_BlockedUsersList.GetHeadPosition();
	pHeadMsg = (CMessagePacket *)m_BlockedUsersList.GetNext(pos);

	while(1)
	{
		if(pHeadMsg != pTailMsg)
			pNextMsg = (CMessagePacket *)m_BlockedUsersList.GetNext(pos);

		// Delete the memory pointed to by pHeadMsg
		delete(pHeadMsg);
		
		// If we ve reached end of list
		if(pHeadMsg == pTailMsg)
			break;

		// Else go ahead 1 position
		pHeadMsg = pNextMsg;
	}

	// Remove all members from list
	m_BlockedUsersList.RemoveAll();
	
	// Now we don't have a blocked users list and will need to download it when needed
	m_bIsBlockedUsersListObtained = false;
}



/**************************************************************************/
// Method Name	: RequestFriendsList
// Description	: Sends a friends list request to the server
// Inputs		: None
// Output		: Message sent to server requesting a friend's list
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::RequestFriendsList()
{
	CMessagePacket msg;
	msg.m_strFrom = m_pMsgPacket->m_strTo;
	msg.m_strReasonFlag = "SL";			
	SendMessagePacket(msg);
}



/**************************************************************************/
// Method Name	: ClearFriendsList
// Description	: Clear friends list from view
// Inputs		: None
// Output		: Clears all names from friend's list tree view
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::ClearFriendsList()
{
	GetViewPtr()->m_FriendsList.DeleteAllItems();
	GetViewPtr()->InitFriendsList();
}



/**************************************************************************/
// Method Name	: ResetTitle
// Description	: Everytime the user logs off / disconnected from server, 
//					we need to reset the doc title using this method
// Inputs		: None
// Output		: Resets the doc title
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::ResetTitle()
{
	SetTitle("Not logged in");
}



/**************************************************************************/
// Method Name	: ApplyPreferenceChanges
// Description	: Called when user clicks Apply or OK button on preferences
//					dialog.
// Inputs		: A flag var indicating which preferences are to be saved.
//				  By default all preferences are saved, if nothing is specified
//				  in this flag.
// Output		: Saves the preferences into registry and calls another method
//					to put them into effect in the running application.
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::ApplyPreferenceChanges(int whichPreferences /*= ALL_PREFERENCES */)
{
	// Get the pointer to app object
	CMessengerClientApp *app = (CMessengerClientApp *)::AfxGetApp();

	// Write preferences to registry

	// Login preferences
	if((whichPreferences == LOGIN_PREFERENCES) || (whichPreferences == ALL_PREFERENCES))
	{
		app->WriteProfileInt("Settings\\Login", "AutoLogin", m_Preferences.m_bAutoLogin);
		app->WriteProfileInt("Settings\\Login", "RememberLogin", m_Preferences.m_bRememberLogin);

		if(m_Preferences.m_bRememberLogin)
		{	
			app->WriteProfileString("Settings\\Login", "UserName", m_Preferences.m_UserName);
			app->WriteProfileString("Settings\\Login", "Password", m_Preferences.m_Password);
		}
		else
		{	
			app->WriteProfileString("Settings\\Login", "UserName", "");
			app->WriteProfileString("Settings\\Login", "Password", "");
		}
	}

	// Color and Fonts preferences
	if((whichPreferences == COlOR_FONTS_PREFERENCES) || (whichPreferences == ALL_PREFERENCES))
	{
		app->WriteProfileInt("Settings\\ColorFonts", "Color", m_Preferences.m_iColor);

		LOGFONT logfont = m_Preferences.m_Font;

		app->WriteProfileInt("Settings\\ColorFonts\\Font", "Height", m_Preferences.m_iFontSize);
		app->WriteProfileInt("Settings\\ColorFonts\\Font", "Width", logfont.lfWidth);
		app->WriteProfileInt("Settings\\ColorFonts\\Font", "Escapement", logfont.lfEscapement);
		app->WriteProfileInt("Settings\\ColorFonts\\Font", "Orientation", logfont.lfOrientation);
		app->WriteProfileInt("Settings\\ColorFonts\\Font", "Weight", logfont.lfWeight);
		app->WriteProfileInt("Settings\\ColorFonts\\Font", "Italic", logfont.lfItalic);
		app->WriteProfileInt("Settings\\ColorFonts\\Font", "Underline", logfont.lfUnderline);
		app->WriteProfileInt("Settings\\ColorFonts\\Font", "Strikeout", logfont.lfStrikeOut);
		app->WriteProfileInt("Settings\\ColorFonts\\Font", "Charset", logfont.lfCharSet);
		app->WriteProfileInt("Settings\\ColorFonts\\Font", "OutPrecision", logfont.lfOutPrecision);
		app->WriteProfileInt("Settings\\ColorFonts\\Font", "ClipPrecision", logfont.lfClipPrecision);
		app->WriteProfileInt("Settings\\ColorFonts\\Font", "Quality", logfont.lfQuality);
		app->WriteProfileInt("Settings\\ColorFonts\\Font", "PitchAndFamily", logfont.lfPitchAndFamily);
		app->WriteProfileString("Settings\\ColorFonts\\Font", "FaceName", logfont.lfFaceName);
		app->WriteProfileInt("Settings\\ColorFonts\\Font", "FontColor", m_Preferences.m_iFontColor);
	}

	// Put these preferences into effect NOW!!
	PutPreferencesIntoEffect(whichPreferences);
}



/**************************************************************************/
// Method Name	: GetApplicationSettingsFromRegistry
// Description	: Used to read application settings from registry
// Inputs		: None
// Output		: Reads application settings from registry 
//					(HKEY_CURRENT_USER - Software - ShirishSoft)
//					and saves them in application preferences
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::GetApplicationSettingsFromRegistry()
{
	// Get doc pointer
	CMessengerClientApp *pApp = (CMessengerClientApp *)::AfxGetApp();

	// Login preferences
	// Check whether the user chose to remember his/her id and password
	m_Preferences.m_bRememberLogin = pApp->GetProfileInt("Settings\\Login", "RememberLogin", 0);

	// Check whether the user chose to auto login
	m_Preferences.m_bAutoLogin = pApp->GetProfileInt("Settings\\Login", "AutoLogin", 0);

	// If the user chose to remember his/her id, get the id and password
	// from registry
	if(m_Preferences.m_bRememberLogin)
	{
		m_Preferences.m_UserName = pApp->GetProfileString("Settings\\Login", "UserName", "");
		m_Preferences.m_Password = pApp->GetProfileString("Settings\\Login", "Password", "");
	}

	// Color and fonts preferences
	m_Preferences.m_iColor = pApp->GetProfileInt("Settings\\ColorFonts", "Color", 0);

	// Create and initialize a LOGFONT structure
	LOGFONT	logfont;
	logfont.lfWidth		= pApp->GetProfileInt("Settings\\ColorFonts\\Font", "Width", 0);
	logfont.lfEscapement  = pApp->GetProfileInt("Settings\\ColorFonts\\Font", "Escapement", 0);
	logfont.lfOrientation = pApp->GetProfileInt("Settings\\ColorFonts\\Font", "Orientation", 0);
	logfont.lfWeight	= pApp->GetProfileInt("Settings\\ColorFonts\\Font", "Weight", 0);
	logfont.lfItalic	= pApp->GetProfileInt("Settings\\ColorFonts\\Font", "Italic", 0);
	logfont.lfUnderline	= pApp->GetProfileInt("Settings\\ColorFonts\\Font", "Underline", 0);
	logfont.lfStrikeOut = pApp->GetProfileInt("Settings\\ColorFonts\\Font", "Strikeout", 0);
	logfont.lfCharSet	= pApp->GetProfileInt("Settings\\ColorFonts\\Font", "Charset", 0);
	logfont.lfOutPrecision = pApp->GetProfileInt("Settings\\ColorFonts\\Font", "OutPrecision", 0);
	logfont.lfClipPrecision = pApp->GetProfileInt("Settings\\ColorFonts\\Font", "ClipPrecision", 0);
	logfont.lfQuality	= pApp->GetProfileInt("Settings\\ColorFonts\\Font", "Quality", 0);
	logfont.lfPitchAndFamily = pApp->GetProfileInt("Settings\\ColorFonts\\Font", "PitchAndFamily", 0);
	strcpy(logfont.lfFaceName, (pApp->GetProfileString("Settings\\ColorFonts\\Font", "FaceName", "")).GetBuffer(0));

	m_Preferences.m_iFontColor = pApp->GetProfileInt("Settings\\ColorFonts\\Font", "FontColor", 0);
	m_Preferences.m_iFontSize	= pApp->GetProfileInt("Settings\\ColorFonts\\Font", "Height", 0);
	m_Preferences.m_strFontFace = logfont.lfFaceName;

	m_Preferences.m_Font = logfont;
}



/**************************************************************************/
// Method Name	: PutPreferencesIntoEffect
// Description	: Used to 'apply' application settings to the running application
//					at runtime
// Inputs		: A flag var indicating which preferences are to be applied.
//				  By default all preferences are applied, if nothing is specified
//				  in this flag.
// Output		: Changes application settings at runtime. These new setting 
//					are read from application preferences object.
//				  If login preferences are to be applied, it makes a logout and
//					relogin with new preferences.
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::PutPreferencesIntoEffect(int whichPreferences /*= ALL_PREFERENCES*/)
{
	// Get the view pointer
	CMessengerClientView *view = GetViewPtr();

	// Login preferences into effect
	// If user has changed user name

	if((whichPreferences == LOGIN_PREFERENCES) || (whichPreferences == ALL_PREFERENCES))
	{
		if((m_Preferences.m_UserName != m_UserName) && m_bLoggedInFlag)
		{
			// Log off and disconnect from server
			SaveModified();

			// Try re connecting to the server
			ConnectToServer();

			// Call the method to connect to server
			view->CallOnFileChangeUser();
		}
	}

	// Color and fonts preferences into effect
	if((whichPreferences == COlOR_FONTS_PREFERENCES) || (whichPreferences == ALL_PREFERENCES))
	{
		UpdateAllViews(NULL, 1);
	}
}



/**************************************************************************/
// Method Name	: ProcessMessageFromOtherUser
// Description	: Called when a message from other user is received
// Inputs		: Message packet received from other user
// Output		: Finds the speak window for the friend, if it's not there, creates it
//				  Puts the message into the speak window.
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::ProcessMessageFromOtherUser(CMessagePacket *m_pMsgPacket)
{
	// See if the window for that friend is open
	m_pSpeakDlg = FindSpeakDialogForFriend(m_pMsgPacket->m_strFrom);

	// If it's not, create one
	if(!m_pSpeakDlg)
	{
		CMessengerClientView *pView = GetViewPtr();

		pView->CreateASpeakWindow(m_pMsgPacket->m_strFrom, m_pMsgPacket->m_strMessageBody.GetBuffer(0), m_pMsgPacket->m_strFontFace, &m_pMsgPacket->m_MessageFont, m_pMsgPacket->m_iFontColor, m_pMsgPacket->m_iFontSize);
	}

	// Copy the text into whole text edit box
	m_pSpeakDlg->AddTextToWholeText(*m_pMsgPacket);
}



/**************************************************************************/
// Method Name	: PopulateBlockedUsersList
// Description	: Used to populate blocked users list
// Inputs		: Message packet received from requester
// Output		: Reads the message list in the msg packet
//					creates a msg packet for each name, puts the name in the
//					from field of that packet
//				  Adds the packet to the blocked users list
// Returns		: void
/**************************************************************************/
void CMessengerClientDoc::PopulateBlockedUsersList(CMessagePacket *pMsg)
{
	// If there's no name in the blocked users list, no need to go further
	if(pMsg->m_msgList.IsEmpty())
		return;

	// Empty the blocked users list already contained here
	EmptyBlockedUsersList();

	while(1)
	{
		// Take the user name from the friends list
		CString str = pMsg->m_msgList.RemoveHead();

		// Create a msg packet
		CMessagePacket *packet = new CMessagePacket();
		packet->m_strFrom = str;

		// Add the msg packet to list
		m_BlockedUsersList.AddTail(packet);

		// Break if we are done with all names
		if(m_pMsgPacket->m_msgList.IsEmpty())
			break;
	}

	// After all this is done, set the flag, so that we know in the
	// future that we have a list of blocked users
	m_bIsBlockedUsersListObtained = true;
}
