//////////////////////////////////////////////////////////////////////
// MessengerClientDoc.h : interface of the CMessengerClientDoc class
// Contains maximum central data and functionality of the application
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MESSENGERCLIENTDOC_H__30B8B550_B68A_11D6_8185_0001031200E7__INCLUDED_)
#define AFX_MESSENGERCLIENTDOC_H__30B8B550_B68A_11D6_8185_0001031200E7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include<fstream.h>

#include "MessengerClientSocket.h"
#include "CommonDialogElements.h"
#include "MessengerDialog.h"
#include "RegisterDlg.h"
#include "MessengerClientView.h"
#include "AddAFriend.h"
#include "AcceptAndAddDlg.h"
#include "ChangePassword.h"
#include "ChildFrm.h"
#include "ListPane.h"
#include "TextPane.h"
#include "DeleteFriendsDlg.h"
#include "Preferences.h"
#include "PreferencesData.h"
#include "BlockedUsersListDlg.h"
#include "SpeakWindow.h"

//#include "OfflineMessagesDlg.h"
#include "OfflineMessagesWindow.h"

#include "MainFrm.h"

class CMessengerClientDoc : public CDocument, public CCommonDialogElements
{
protected: // create from serialization only
	CMessengerClientDoc();
	DECLARE_DYNCREATE(CMessengerClientDoc)

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMessengerClientDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	protected:
	virtual BOOL SaveModified();
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CMessengerClientDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

	public:
	// This user's login info
	CString	m_UserName;
	CString	m_Password;

	// Titles for the friends tree shown in the view
	HTREEITEM m_hFriends;		// "Friends list"
	HTREEITEM m_hOnline;		// "Online Friends"
	HTREEITEM m_hOffline;		// "Offline Friends"

	// Number of online and offline friends
	int m_nNumOnline, m_nNumOffline;

	// List of all the dialogs opened for speaking to other users
	CPtrList  m_SpeakDialogsList;

	// List of all users permanently blocked by this user
	CPtrList  m_BlockedUsersList;
	bool m_bIsBlockedUsersListObtained;	// true when blocked member's list is present here

	// Current speaking dialog
	CSpeakWindow *m_pSpeakDlg;

	// Login dialog
	CMessengerDialog *m_pMessengerLoginDlg;

	// Add Friend dialog
	CAddAFriend *m_pAddAFriendDlg;

	// Accept and Add dialog
	CAcceptAndAddDlg *m_pAcceptAndAddDlg;

	// Offline messages dialog
	COfflineMessagesWindow *m_pOfflineMessagesDlg;

	// Change password dialog
	CChangePassword *m_pChangePasswordDlg;

	// Offline messages list
	CPtrList		m_OfflineMessagesList;

	// Delete Friends dialog
	CDeleteFriendsDlg *m_DeleteFriendsDlg;

	// Preferences Dialog
	CPreferences	  *m_pPreferencesDlg;

	// Blocked users dialog
	CBlockedUsersListDlg	*m_pBlockedUsersDlg;

	// Preferences data
	CPreferencesData	m_Preferences;

	static BOOL m_bConnectedFlag;	// TRUE if socket open to the server
	BOOL m_bLoggedInFlag;			// TRUE if this user is logged in

	fstream m_fOfflineMessagesFile;		// We will write offline messages thru this file handle

// Generated message map functions
protected:
	//{{AFX_MSG(CMessengerClientDoc)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

	// Generated OLE dispatch map functions
	//{{AFX_DISPATCH(CMessengerClientDoc)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_DISPATCH
	DECLARE_DISPATCH_MAP()
	DECLARE_INTERFACE_MAP()
public:
	CSpeakWindow *FindSpeakDialogForFriend(CString strFriend);
	void ApplyPreferenceChanges(int whichPreferences = ALL_PREFERENCES);
	void DoCleanup();
	BOOL ConnectToServer();
	void DisconnectFromServer();
	virtual bool ProcessMessagePacket();
	void RemoveSpeakDialogFromList(CString strFriend);
	CMessengerClientView *GetViewPtr();

	void CreateAnOfflineMessagesDialog();
protected:
	void PopulateBlockedUsersList(CMessagePacket *pMsg);
	void EmptyBlockedUsersList();
	void AddFriendToList(CMessagePacket *m_pMsgPacket, BOOL online);
	void ClearFriendsList();
	void EmptyAllLists();
	void EmptyOfflineMessagesList();
	void EmptySpeakDialogsList();
	HTREEITEM FindFriendFromList(CMessagePacket *m_pMsgPacket);
	void GetApplicationSettingsFromRegistry();
	void InsertMessagesIntoOfflineMessagesDialog(COfflineMessagesFrameWnd *pFrame);
	void ProcessMessageFromOtherUser(CMessagePacket *m_pMsgPacket);
	void PutPreferencesIntoEffect(int whichPreferences = ALL_PREFERENCES);
	void RemoveFriendFromList(CMessagePacket *m_pMsgPacket, HTREEITEM& user);
	void RequestFriendsList();
	void ResetTitle();
	void ShowFriendsList(CMessagePacket *m_pMsgPacket);
	void ShowOnlineOfflineSummary();
	HTREEITEM TraverseTree(HTREEITEM root, CMessengerClientView *pView);
	void WriteOffLineMessageToDatabase(CMessagePacket *pMsgPacket);
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MESSENGERCLIENTDOC_H__30B8B550_B68A_11D6_8185_0001031200E7__INCLUDED_)
