// MessengerDialog.cpp : implementation file
//

#include "stdafx.h"
#include "MessengerClient.h"
#include "MessengerDialog.h"
#include "RegisterDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMessengerDialog dialog


CMessengerDialog::CMessengerDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CMessengerDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMessengerDialog)
	m_UserName = _T("");
	m_Password = _T("");
	m_bAutoLogin = FALSE;
	m_bRememberLogin = FALSE;
	//}}AFX_DATA_INIT
}


CMessengerDialog::CMessengerDialog(CMessengerClientDoc *pDoc, CWnd* pParent /*= NULL*/)  // additional constructor
//CMessengerDialog::CMessengerDialog(CCommonDialogElements *pDoc, CWnd* pParent /*= NULL*/)   // additional constructor
	: CDialog(CMessengerDialog::IDD, pParent)
{
	m_Password = _T("");
	m_UserName = _T("");
	m_bAutoLogin = FALSE;
	m_bRememberLogin = FALSE;

	m_pParent = pDoc;
	m_bDoLogin = true;
}


void CMessengerDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMessengerDialog)
	DDX_Control(pDX, IDOK, m_btnOK);
	DDX_Text(pDX, IDC_USERNAME, m_UserName);
	DDX_Text(pDX, IDC_PASSWORD, m_Password);
	DDX_Check(pDX, IDC_AUTO_LOGIN, m_bAutoLogin);
	DDX_Check(pDX, IDC_REMEMBER_LOGIN, m_bRememberLogin);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMessengerDialog, CDialog)
	//{{AFX_MSG_MAP(CMessengerDialog)
	ON_BN_CLICKED(IDC_REGISTER, OnRegister)
	ON_EN_CHANGE(IDC_USERNAME, OnChangeUsername)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMessengerDialog message handlers




/**************************************************************************/
// Method Name	: OnOK
// Description	: This method is called when user clicks Login button
// Output		: Verifies the user inputs and sends the login request to the server
//				  If input is wrong, shows appropriate error messages
//				  After sending the Login request to the server, this dialog is
//				  hidden to block any further input from user, and reappears when 
//				  it gets response from server.
// Returns		: void
/**************************************************************************/
void CMessengerDialog::OnOK() 
{
	// Get values from screen
	UpdateData(TRUE);
	CMessagePacket msgPacket;

	int ret = IsPasswordValid(m_Password);

	switch(ret)
	{
	case 0:
		// Verify Login name as well
		ret = IsPasswordValid(m_UserName);

		switch(ret)
		{
		case 0:
			// First log off if already logged in
			if(m_pParent->m_bLoggedInFlag == TRUE)
			{
				m_pParent->DoCleanup();

				// Since logging off breaks our connection with the server, reconnect
				// But before that do some cleanup
				m_pParent->DisconnectFromServer();
				m_pParent->ConnectToServer();
			}

			// Initialize a message packet for login process
			msgPacket.m_strFrom = m_UserName;
			msgPacket.m_strTo = m_Password;
			msgPacket.m_strReasonFlag = "L";	// Because we are logging in here
			msgPacket.m_msgList.AddTail(msgPacket.m_strFrom);
			msgPacket.m_msgList.AddTail(msgPacket.m_strTo);
			msgPacket.m_msgList.AddTail(msgPacket.m_strReasonFlag);

			// Send some data on the connected socket
			SendMessagePacket(msgPacket);

			// Hide the dialog
			this->ShowWindow(SW_HIDE);
			//CDialog::OnOK();

			break;
		default:
			::AfxMessageBox("Invalid Login Name");
		}

		break;
	case 1:				// Password contains spaces
		::AfxMessageBox("Password cannot contain spaces");
		break;

	case 2:				// Password length incorrect
		::AfxMessageBox("Password length must be 1-14 characters");
		break;

	case 3:
		::AfxMessageBox("Password can contain only the characters from A-Z, a-z and 0-9");
		break;
	}
}



/**************************************************************************/
// Method Name	: OnCancel
// Description	: This method is called when user clicks Cancel button
// Output		: Closes the dialog
// Returns		: void
/**************************************************************************/
void CMessengerDialog::OnCancel() 
{
	CDialog::OnCancel();
}



/**************************************************************************/
// Method Name	: OnRegister
// Description	: This method is called when user clicks Register button
// Output		: Invokes registration dialog
// Returns		: void
/**************************************************************************/
void CMessengerDialog::OnRegister() 
{
	m_pRegisterDlg = new CRegisterDlg(this, NULL);
	m_pRegisterDlg->DoModal();	

	// Free memory
	delete(m_pRegisterDlg);
	m_pRegisterDlg = NULL;
}



/**************************************************************************/
// Method Name	: IsPasswordValid
// Description	: This method verifies the password typed by the user
// Inputs		: Password typed by the user
// Output		: None
// Returns		: 1	- Password contains spaces
//				  2 - Password is too long or null
//				  3 - Password contains invalid chars (anything except A-Za-z0-9)
//				  0 - Everything is all right
// Note			: When making changes to this method, also take care to make similar
//				  changes in CChangePassword::IsPasswordValid and
//				  CRegisterDlg::IsPasswordValid methods
/**************************************************************************/
int CMessengerDialog::IsPasswordValid(CString password)
{
	// Eliminate the spaces on left and right of password
	CString tstPass = password;
	tstPass.TrimLeft();
	tstPass.TrimRight(' ');

	if(tstPass != password)		// Password contains spaces on left and/or right
		return 1;
	
	// If password is more than 14 chars long or zero
	int len = password.GetLength();
	if(len == 0 || len > MAX_PASSWORD_LENGTH)
		return 2;


	// If password contains anything except A-Z, a-z and 0-9
	// If password contains anything except A-Z, a-z and 0-9
	for(int i = 0; i < len; i++)
	{
		char ch = password.GetAt(i);
		if((ch < 'A' || ch > 'Z') && (ch < 'a' || ch > 'z') && (ch < '0' || ch > '9'))
			return 3;
	}

	return 0;
}



/**************************************************************************/
// Method Name	: IsPasswordValid
// Description	: This method sends 1 message packet to the server
// Inputs		: Message packet to be sent to the server
// Output		: Sends message packet thru output archive
// Returns		: TRUE - Success
//				  FALSE - Failure
/**************************************************************************/
bool CMessengerDialog::SendMessagePacket(CMessagePacket& msgPacket)
{
	TRY
	{
		msgPacket.Serialize(*(m_pParent->m_pArOut));
		m_pParent->m_pArOut->Flush();
	}
	CATCH(CFileException, e)
	{
		return FALSE;
	}
	END_CATCH

	return TRUE;
}



/**************************************************************************/
// Method Name	: OnInitDialog
// Description	: This method initializes the messenger dialog
// Inputs		: None
// Output		: Calls base class method
//				  Initializes preferences and other dialog vars
//				  If user has chosen autologin, does the login automatically
// Returns		: TRUE - Success
//				  FALSE - Failure
/**************************************************************************/
BOOL CMessengerDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Get registry settings
	CMessengerClientApp *app = (CMessengerClientApp *)::AfxGetApp();

	// Get preferences from doc object
	CPreferencesData preferences;
	preferences = m_pParent->m_Preferences;

	// Copy the login preferences here
	m_bRememberLogin	= preferences.m_bRememberLogin;
	m_UserName			= preferences.m_UserName;
	m_Password			= preferences.m_Password;
	m_bAutoLogin		= preferences.m_bAutoLogin;

	// Put these on the screen
	UpdateData(FALSE);

	// If the user chose to auto login, check the auto login box
	// and send these to server for logging in
	if(m_bAutoLogin && m_bDoLogin)
	{
		m_btnOK.EnableWindow(FALSE);
		OnOK();
	}

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}



/**************************************************************************/
// Method Name	: OnChangeUsername
// Description	: Called when user types in a user name
// Inputs		: None
// Output		: Enables the Login button. If user has chosen autologin
//					and that user name is already logged in, this will help to 
//					allow login with a diff name
// Returns		: void
/**************************************************************************/
void CMessengerDialog::OnChangeUsername() 
{
	m_btnOK.EnableWindow();
}
