// OfflineMessagesFrameWnd.cpp : implementation file
//

#include "stdafx.h"
#include "MessengerClient.h"
#include "OfflineMessagesFrameWnd.h"
#include "TextPane.h"
#include "MessengerClientDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "ListPane.h"

/////////////////////////////////////////////////////////////////////////////
// COfflineMessagesFrameWnd

IMPLEMENT_DYNCREATE(COfflineMessagesFrameWnd, CFrameWnd)

COfflineMessagesFrameWnd::COfflineMessagesFrameWnd()
{
}

COfflineMessagesFrameWnd::~COfflineMessagesFrameWnd()
{
}


BEGIN_MESSAGE_MAP(COfflineMessagesFrameWnd, CFrameWnd)
	//{{AFX_MSG_MAP(COfflineMessagesFrameWnd)
	ON_COMMAND(ID_FILE_DELETE, OnFileDelete)
	ON_COMMAND(ID_FILE_REPLY, OnFileReply)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COfflineMessagesFrameWnd message handlers




/**************************************************************************/
// Method Name	: OnCreateClient
// Description	: Called internally while creating the frame window
// Inputs		: System defined
// Output		: Creates a static splitter window inside the frame and loads
//				  2 panes. First pane stores all offline messages listwise,
//				  the second pane contains a text box which shows the 
//				  message body of the message selected in the first pane
// Returns		: TRUE - Success
//				  FALSE - otherwise
/**************************************************************************/
BOOL COfflineMessagesFrameWnd::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext) 
{
	if(!m_wndSplitter.CreateStatic(this, 2, 1))
		return FALSE;

	float nPaneHt = OFFLINE_MESSAGES_DIALOG_HEIGHT * 0.5;
	m_ListPane = (CListPane *)m_wndSplitter.CreateView(0, 0, RUNTIME_CLASS(CListPane), CSize(OFFLINE_MESSAGES_DIALOG_WIDTH, nPaneHt), pContext);
	if(!m_ListPane)
		return FALSE;

	nPaneHt = OFFLINE_MESSAGES_DIALOG_HEIGHT * 0.5;
	m_TextPane = (CTextPane *)m_wndSplitter.CreateView(1, 0, RUNTIME_CLASS(CTextPane), CSize(OFFLINE_MESSAGES_DIALOG_WIDTH, nPaneHt), pContext);	
	if(!m_TextPane)
		return FALSE;

	return TRUE;
}



/**************************************************************************/
// Method Name	: OnFileDelete
// Description	: Method to call the delete method of Text buttons pane object
// Inputs		: void
// Output		: Selected messages are deleted
// Returns		: void
/**************************************************************************/
void COfflineMessagesFrameWnd::OnFileDelete() 
{
	m_TextPane->OnFileDelete();	
}



/**************************************************************************/
// Method Name	: OnFileReply
// Description	: Method to call the reply method of Text buttons pane object
// Inputs		: void
// Output		: Speak window created fro selected message
// Returns		: void
/**************************************************************************/
void COfflineMessagesFrameWnd::OnFileReply() 
{
	m_TextPane->OnFileReply();	
}



/**************************************************************************/
// Method Name	: DestroyWindow
// Description	: Called internally while destroying the frame window
// Inputs		: None
// Output		: Notifies the doc object abt closing and calls base class method
// Returns		: TRUE - Success
//				  FALSE - otherwise
/**************************************************************************/
BOOL COfflineMessagesFrameWnd::DestroyWindow() 
{
	// Get view pointer
	//COfflineMessagesDlg *pView = (COfflineMessagesDlg *)GetActiveView();
	COfflineMessagesWindow *pView = (COfflineMessagesWindow *)GetActiveView();

	// Tell the Doc object this window is closing
	((CMessengerClientDoc *)pView->GetDocument())->m_pOfflineMessagesDlg = NULL;
	
	return CFrameWnd::DestroyWindow();
}



/**************************************************************************/
// Method Name	: OnFileExit
// Description	: Method called when File-Exit menu item is clicked
// Inputs		: void
// Output		: Offline messages window is closed
// Returns		: void
/**************************************************************************/
void COfflineMessagesFrameWnd::OnFileExit() 
{
	// Call the default message handler to close the window
	OnClose();
}
