// RegisterDlg.cpp : implementation file
//

#include "stdafx.h"
#include "MessengerClient.h"
#include "RegisterDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

class CMessengerDialog;

/////////////////////////////////////////////////////////////////////////////
// CRegisterDlg dialog


CRegisterDlg::CRegisterDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRegisterDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CRegisterDlg)
	m_Password = _T("");
	m_Password2 = _T("");
	m_Username = _T("");
	//}}AFX_DATA_INIT
}

CRegisterDlg::CRegisterDlg(CMessengerDialog *pDoc, CWnd* pParent /*= NULL*/)   // additional constructor
	: CDialog(CRegisterDlg::IDD, pParent)
{
	m_Password = _T("");
	m_Password2 = _T("");
	m_Username = _T("");

	m_pParent = pDoc;
}


void CRegisterDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRegisterDlg)
	DDX_Text(pDX, IDC_PASSWORD, m_Password);
	DDX_Text(pDX, IDC_PASSWORD2, m_Password2);
	DDX_Text(pDX, IDC_USERNAME, m_Username);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CRegisterDlg, CDialog)
	//{{AFX_MSG_MAP(CRegisterDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRegisterDlg message handlers




/**************************************************************************/
// Method Name	: OnOK
// Description	: This sends the regitration request to the server
// Inputs		: None
// Output		: Validates password
//				  If password is invalid, shows appropriate messages
//				  Sends registration request to the server
// Returns		: void
/**************************************************************************/
void CRegisterDlg::OnOK() 
{
	// Get values from screen
	UpdateData(TRUE);

	int ret = IsPasswordValid(m_Password, m_Password2);

	switch(ret)
	{
	case 0:				// Password is all right, check login
		ret = IsPasswordValid(m_Username, "");

		switch(ret)
		{
		case 0:
			// Create and initialize a message packet
			m_MsgPacket.m_strFrom = m_Username;
			m_MsgPacket.m_strTo = m_Password;
			m_MsgPacket.m_strReasonFlag = "R";	// Because we are registering in here

			// Send some data on the connected socket
			m_pParent->SendMessagePacket(m_MsgPacket);
			break;
		default:
			::AfxMessageBox("Invalid Login Name");
		}
		break;

	case 1:				// password entries do not match
		::AfxMessageBox("Password entries much match");
		break;

	case 2:				// Password contains spaces
		::AfxMessageBox("Password cannot contain spaces");
		break;

	case 3:				// Password length incorrect
		::AfxMessageBox("Password length must be 1-14 characters");
		break;

	case 4:
		::AfxMessageBox("Password can contain only the characters from A-Z, a-z and 0-9");
		break;
	}

	// Call base class functionality
	//CDialog::OnOK();
}



/**************************************************************************/
// Method Name	: OnCancel
// Description	: Closes the registration dialog
// Inputs		: None
// Output		: Registration dialog closed
// Returns		: void
/**************************************************************************/
void CRegisterDlg::OnCancel() 
{
	CDialog::OnCancel();
}



/**************************************************************************/
// Method Name	: IsPasswordValid
// Description	: This method verifies the password typed by the user
// Inputs		: Password typed by the user
// Output		: None
// Returns		: 1	- Password entris don't match
//				  2 - Password contains spaces
//				  3 - Password is too long or null
//				  4 - Password contains invalid chars (anything except A-Za-z0-9)
//				  0 - Everything is all right
// Note			: When making changes to this method, also take care to make similar
//				  changes in CChangePassword::IsPasswordValid and
//				  CMessengerDialog::IsPasswordValid methods
/**************************************************************************/
int CRegisterDlg::IsPasswordValid(CString password, CString password2 /* = NULL*/)
{
	if(password2 != "")		// We are checking password, not login
	{
		// Check whether the password entries match
		if(password != password2)
			return 1;
	}

	// Eliminate the spaces on left and right of password
	CString tstPass = password;
	tstPass.TrimLeft();
	tstPass.TrimRight(' ');

	if(tstPass != password)		// Password contains spaces on left and/or right
		return 2;
	
	// If password is more than 14 chars long or zero
	int len = password.GetLength();
	if(len == 0 || len > MAX_PASSWORD_LENGTH)
		return 3;


	// If password contains anything except A-Z, a-z and 0-9
	for(int i = 0; i < len; i++)
	{
		char ch = password.GetAt(i);
		if((ch < 'A' || ch > 'Z') && (ch < 'a' || ch > 'z') && (ch < '0' || ch > '9'))
			return 4;
	}

	return 0;
}
