// SpeakWindow.cpp : implementation file
//

#include "stdafx.h"
#include "MessengerClient.h"
#include "SpeakWindow.h"
#include "MessengerClientDoc.h"
#include "MessengerClientView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSpeakWindow

IMPLEMENT_DYNCREATE(CSpeakWindow, CFormView)

CSpeakWindow::CSpeakWindow()
	: CFormView(CSpeakWindow::IDD)
{
	//{{AFX_DATA_INIT(CSpeakWindow)
	m_strFrom = _T("");
	m_strTo = _T("");
	m_strTypedText = _T("");
	m_strWholeText = _T("");
	//}}AFX_DATA_INIT

	m_bFirstTime = true;
	m_bNewSpeakWindow = true;
	m_iWholeTextFontColor = -1;
	m_iWholeTextFontSize = -1;
	m_bDirty = false;
	m_bSavedOnce = false;
	m_strFileName = "";
}

CSpeakWindow::~CSpeakWindow()
{
}

void CSpeakWindow::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSpeakWindow)
	DDX_Control(pDX, IDC_BACKCOLOR, m_btnBackColor);
	DDX_Control(pDX, IDC_FONT, m_btnFont);
	DDX_Control(pDX, IDC_TO, m_ctrlTo);
	DDX_Control(pDX, IDC_SEND, m_btnSend);
	DDX_Control(pDX, IDC_WHOLETEXT, m_ctrlWholeText);
	DDX_Control(pDX, IDC_TYPEDTEXT, m_ctrlTypedText);
	DDX_Text(pDX, IDC_FROM, m_strFrom);
	DDX_Text(pDX, IDC_TO, m_strTo);
	DDX_Text(pDX, IDC_TYPEDTEXT, m_strTypedText);
	DDX_Text(pDX, IDC_WHOLETEXT, m_strWholeText);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSpeakWindow, CFormView)
	//{{AFX_MSG_MAP(CSpeakWindow)
	ON_BN_CLICKED(IDC_SEND, OnSend)
	ON_WM_SIZE()
	ON_WM_GETMINMAXINFO()
	ON_BN_CLICKED(IDC_FONT, OnFont)
	ON_BN_CLICKED(IDC_BACKCOLOR, OnBackcolor)
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
	ON_COMMAND(ID_FILE_SEND, OnFileSend)
	ON_COMMAND(ID_FILE_CLOSE, OnFileClose)
	ON_COMMAND(ID_FORMAT_FONT, OnFormatFont)
	ON_COMMAND(ID_FORMAT_COLOR, OnFormatColor)
	ON_COMMAND(ID_FORMAT_DEFAULTFONTANDCOLOR, OnFormatDefaultfontandcolor)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_COMMAND(ID_EDIT_DELETE, OnEditDelete)
	ON_COMMAND(ID_EDIT_SELECTALL, OnEditSelectall)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_FILE_SAVEAS, OnFileSaveas)
	ON_WM_CREATE()
	ON_WM_KILLFOCUS()
	ON_WM_SETFOCUS()
	ON_COMMAND(ID_FILE_BLOCKTHISUSER, OnFileBlockthisuser)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSpeakWindow diagnostics

#ifdef _DEBUG
void CSpeakWindow::AssertValid() const
{
	CFormView::AssertValid();
}

void CSpeakWindow::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSpeakWindow message handlers




/**************************************************************************/
// Method Name	: SetTextFont
// Description	: Sets the font of entire rich text control or some part of
//				  its text
// Inputs		: New Font for the text box
//				  Font face name
//				  Reference of the Rich Text box where changes are to be made
//				  New font color
//				  New font size
//				  How much text will be affected by this? (Entire text, some text or
//					the newly appended text only)
// Output		: According to the font sent, initializes a CHARFORMAT structure
//				  and applies it to the rich edit control text.
// Returns		: void
/**************************************************************************/
void CSpeakWindow::SetTextFont(LOGFONT& font, CString& strFontFace, CRichEditCtrl& control, int iFontColor, int iFontSize, int iHowMuchText)
{
	// If user hasn't set the font preferences, no need to go further
	if(!(strFontFace.GetLength()))
		return;

	CHARFORMAT format;

	// Initialize the char format to nulls
	InitCharFormat(format);

	strcpy(format.szFaceName, strFontFace.GetBuffer(0));
	format.dwMask = format.dwMask | CFM_ITALIC| CFM_STRIKEOUT| CFM_UNDERLINE| CFM_BOLD;

	// Test if italic is selected
	if(font.lfItalic)
		format.dwEffects = format.dwEffects | CFE_ITALIC;

	// If Strikeout is selected
	if(font.lfStrikeOut)
		format.dwEffects = format.dwEffects | CFE_STRIKEOUT;

	// If Underline is selected
	if(font.lfUnderline)
		format.dwEffects = format.dwEffects | CFE_UNDERLINE;

	// If Bold is selected
	if(font.lfWeight == FW_BOLD)
		format.dwEffects = format.dwEffects | CFE_BOLD;

	// Adjust the char height
	format.dwMask = format.dwMask | CFM_SIZE;
	format.yHeight = iFontSize;

	// Adjust pitch and family, charset
	format.bPitchAndFamily  = font.lfPitchAndFamily;
	format.bCharSet			= font.lfCharSet;

	// Adjust the font color
	format.dwMask = format.dwMask | CFM_COLOR;
	format.crTextColor = iFontColor;

	// Set this as the control's char format, or the selected text, depending on the flag
	if(iHowMuchText == FORMAT_ALL_TEXT)
		control.SetDefaultCharFormat(format);

	else
	{
		// Get the total length of the text
		int nTotalLen = m_strWholeText.GetLength();

		// Select the last char, which is /r/n. This is just to ensure ReplaceSel call
		// doesn't go wrong
		control.SetSel(nTotalLen - 1, nTotalLen);

		// Set the new format for selection
		control.SetSelectionCharFormat(format);
	}
}



/**************************************************************************/
// Method Name	: GetCurrentFont
// Description	: Gets the current font settings and stores the corresponding 
//				  LOGFONT structure into m_Font var. 
// Inputs		: A Message packet
// Output		: If a valid font is stored in preferences, stores it as the 
//				  message font. Otherwise does default settings.
// Returns		: void
/**************************************************************************/
void CSpeakWindow::GetCurrentFont(CMessagePacket& msg)
{
	LOGFONT font = m_Preferences.m_Font;
	if(strcmp(font.lfFaceName, ""))
	{
		msg.m_strFontFace.Empty();
		msg.m_strFontFace.Insert(0, font.lfFaceName);
		//msg.m_strFontFace = font.lfFaceName;
		msg.m_MessageFont = font;
		msg.m_iFontColor  = m_Preferences.m_iFontColor;
		msg.m_iFontSize  = m_Preferences.m_iFontSize;
	}
	else
	{
		msg.m_strFontFace = "Arial";
		msg.m_MessageFont = font;
		msg.m_iFontColor  = 0;
		msg.m_iFontSize  = 10;
	}
}



/**************************************************************************/
// Method Name	: InitCharFormat
// Description	: Initializes a CHARFORMAT structure
// Inputs		: A CHARFORMAT var
// Output		: Initialized CHARFORMAT
// Returns		: void
/**************************************************************************/
void CSpeakWindow::InitCharFormat(CHARFORMAT &format)
{
	format.dwMask = 0;
	format.dwEffects = 0;
	format.yHeight = 0;
	format.yOffset = 0;
	format.crTextColor = 0;
	format.bCharSet = 0;
	format.bPitchAndFamily = 0;
	strcpy(format.szFaceName, "");
}



/**************************************************************************/
// Method Name	: AddTextToWholeText
// Description	: Copies the typed text / sent text to the whole text rich edit box
// Inputs		: A Message packet
// Output		: The new text is put into the Whole Text rich edit control, either
//				  appended (if speak window already exists) or just put in there 
//				  (if speak window is just created and whole text box is empty)
// Returns		: void
/**************************************************************************/
void CSpeakWindow::AddTextToWholeText(CMessagePacket &msg)
{
	// This string will be appended to the old string in whole text
	CString m_strAddString = msg.m_strFrom + ": ";

	m_strAddString += msg.m_strMessageBody;

	// Workaround to solve problem where an extra newline is inserted
	// in this text box after pressing Send button.
	if(!m_bNewSpeakWindow)
		// We are overwriting Newline in the previous text, so compensate for it
		//m_strAddString.Insert(0, "\r\n");
		m_strAddString.Insert(0, "\n");

	// Populate the speak window background variables
	UpdateData(TRUE);

	// Do this addition to the text in the background, too, so that both are
	// consistent
	if(m_bNewSpeakWindow)		// If new window
	{
		m_strWholeText = m_strAddString;

		// Set this font to whole text rich edit control, if this box is newly opened
		SetTextFont(msg.m_MessageFont, msg.m_strFontFace, m_ctrlWholeText, msg.m_iFontColor, msg.m_iFontSize, FORMAT_ALL_TEXT);
	}
	else
	{
		m_strWholeText +=  m_strAddString;

		// Set this font to the new text added to text rich edit control, if this box is 
		// already there
		SetTextFont(msg.m_MessageFont, msg.m_strFontFace, m_ctrlWholeText, msg.m_iFontColor, msg.m_iFontSize, FORMAT_APPEND_TEXT);
	}

	// Put this text into rich text box
	m_ctrlWholeText.ReplaceSel(m_strAddString.GetBuffer(0));

	m_bNewSpeakWindow = false;
	m_bDirty	= true;
}



/**************************************************************************/
// Method Name	: OnInitialUpdate
// Description	: Called when window is being initalized
// Inputs		: None
// Output		: Local copy of preferences is made.
//				  Current font settings are retrived and Typing Text box is
//				  initilized with current font in the preferences
//				  Form controls are resized and placed in appropriate positions.
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnInitialUpdate() 
{
	CFormView::OnInitialUpdate();

	// Set parent
	m_pParent = (CMessengerClientDoc *)GetDocument();
	
	// Set the speak dialog's preferences
	m_Preferences = m_pParent->m_Preferences;

	CMessagePacket msg;		// Dummy, just to satisfy the interface

	// Get the font preferences
	GetCurrentFont(msg);

	// Set this font to typed text rich text box
	SetTextFont(msg.m_MessageFont, msg.m_strFontFace, m_ctrlTypedText, msg.m_iFontColor, msg.m_iFontSize, FORMAT_ALL_TEXT);

	// Adjust the control positions on the form
	AdjustFormControls();
	
	// Set background color
	SetControlsBackColor(m_Preferences.m_iColor);
}



/**************************************************************************/
// Method Name	: OnSend
// Description	: Called when user presses Send button
// Inputs		: None
// Output		: If the user has not typed anything into typing box, shows
//					an error message
//				  Otherwise copies the typed text into whole text box
//				  Sends the message to server.
//				  Clears the typd text box.
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnSend() 
{
	// Get the data from screen into the vars
	UpdateData(TRUE);	

	if(m_strTo.GetLength() == 0)
	{
		::AfxMessageBox("Please enter a To party ID");
		return;
	}

	if(m_strTypedText.GetLength() == 0)
		return;

	// Create a message packet
	CMessagePacket msg;
	msg.m_strFrom	= m_strFrom;
	msg.m_strTo		= m_strTo;
	msg.m_strReasonFlag = "M";
	msg.m_strMessageBody = m_strTypedText;

	// Get the current system font into the message packet
	GetCurrentFont(msg);

	// Put the new text into the whole text rich edit box
	AddTextToWholeText(msg);

	// Send the message packet
	m_pParent->SendMessagePacket(msg);
	
	// Erase text from typed text box
	m_ctrlTypedText.SetSel(0, -1);
	m_ctrlTypedText.ReplaceSel("");
}



/**************************************************************************/
// Method Name	: OnSize
// Description	: Called when window resizes
// Inputs		: New window coordinates, system defined
// Output		: Calls the base class method
//				  If it's the first time window is being resized, calls method
//				  to resize and adjust controls
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnSize(UINT nType, int cx, int cy) 
{
	CFormView::OnSize(nType, cx, cy);

	// Do this only if the window is resized after display
	if(!m_bFirstTime)
		// Resize controls appropriately and place them on proper positions
		AdjustFormControls();
}



/**************************************************************************/
// Method Name	: AdjustFormControls
// Description	: Method to put all controls on the form in their proper places
//				  Called from various places
// Inputs		: void
// Output		: All controls on the form resized and adjusted
// Returns		: void
/**************************************************************************/
void CSpeakWindow::AdjustFormControls()
{
	// These rects will contain the coordinates of speak window
	// and that of various controls
	CRect rectWin, rectSendButton, rectTypedText, rectWholeText, rectTo, rectFormatControl;

	// Get main window's rects
	GetWindowRect(&rectWin);

	/****************************************************************************/
	// Position the Send button
	/****************************************************************************/
	// Get Send button's rect
	m_btnSend.GetWindowRect(&rectSendButton);

	// Adjust the Send button's position to the bottom-right side of window
	int htButton = rectSendButton.Height();
	int wdButton = rectSendButton.Width();
	rectSendButton.left	= rectWin.right - wdButton - BUFFER_DISTANCE;
	rectSendButton.top	= rectWin.bottom - htButton - BUFFER_DISTANCE;
	rectSendButton.right  = rectWin.right - BUFFER_DISTANCE;
	rectSendButton.bottom = rectWin.bottom - BUFFER_DISTANCE;

	ScreenToClient(&rectSendButton);
	m_btnSend.MoveWindow(rectSendButton.left, rectSendButton.top, wdButton, htButton);

	/****************************************************************************/
	// Position the Typed Text box
	/****************************************************************************/
	// Get typed text rich edit control's rect
	m_ctrlTypedText.GetWindowRect(&rectTypedText);

	// Adjust the typed text rich edit box's position
	int htTypedText = rectTypedText.Height();
	int wdTypedText = rectTypedText.Width();
	rectTypedText.left	= rectWin.left + BUFFER_DISTANCE;
	rectTypedText.top	= rectWin.bottom - htTypedText - BUFFER_DISTANCE;
	rectTypedText.right  = rectWin.right - (2 * BUFFER_DISTANCE) - wdButton;
	rectTypedText.bottom = rectWin.bottom - BUFFER_DISTANCE;

	// Recalculate height and width
	htTypedText = rectTypedText.Height();
	wdTypedText = rectTypedText.Width();

	ScreenToClient(&rectTypedText);
	m_ctrlTypedText.MoveWindow(rectTypedText.left, rectTypedText.top, wdTypedText, htTypedText);

	/****************************************************************************/
	// Position the Whole Text box
	/****************************************************************************/
	// Get whole text rich edit control's rect and To box
	m_ctrlWholeText.GetWindowRect(&rectWholeText);
	m_ctrlTo.GetWindowRect(&rectTo);

	// Adjust the whole text rich edit box's position
	rectWholeText.left	= rectWin.left + BUFFER_DISTANCE;
	rectWholeText.top	= rectWin.top + (2 * BUFFER_DISTANCE) + rectTo.Height();
	rectWholeText.right  = rectWin.right - BUFFER_DISTANCE;
	rectWholeText.bottom = rectWin.bottom - (3 * BUFFER_DISTANCE) - htTypedText - SPEAK_WINDOW_TOOLBAR_HEIGHT;
	int htWholeText = rectWholeText.Height();
	int wdWholeText = rectWholeText.Width();

	/****************************************************************************/
	// Position the Format bar controls
	/****************************************************************************/
	////////// FONT Button
	rectFormatControl.left = rectWholeText.left;
	rectFormatControl.top = rectWholeText.bottom + BUFFER_DISTANCE;
	rectFormatControl.right = rectFormatControl.left + BTN_WIDTH;
	rectFormatControl.bottom = rectFormatControl.top + BTN_HEIGHT;
	ScreenToClient(&rectFormatControl);
	m_btnFont.MoveWindow(&rectFormatControl);
	////////// Background color button
	rectFormatControl.left += BTN_WIDTH + SPEAK_WINDOW_TOOLBAR_CONTROLS_DISTANCE;
	rectFormatControl.right = rectFormatControl.left + BTN_WIDTH;
	m_btnBackColor.MoveWindow(&rectFormatControl);


	// Recalculate height and width
	htWholeText = rectWholeText.Height();
	wdWholeText = rectWholeText.Width();

	ScreenToClient(&rectWholeText);
	m_ctrlWholeText.MoveWindow(rectWholeText.left, rectWholeText.top, wdWholeText, htWholeText);

	// Set this var to false, because window is displayed once
	m_bFirstTime = false;
}



/**************************************************************************/
// Method Name	: OnGetMinMaxInfo
// Description	: Method to control window sizing
// Inputs		: System defined
// Output		: 
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI) 
{
	// Don't let the window get below a certain size
	lpMMI->ptMinTrackSize.x = SPEAK_WINDOW_MIN_ALLOWED_WIDTH;
	lpMMI->ptMinTrackSize.y = SPEAK_WINDOW_MIN_ALLOWED_HEIGHT;
	
	CFormView::OnGetMinMaxInfo(lpMMI);
}



/**************************************************************************/
// Method Name	: OnFont
// Description	: Calld when User presses Font button on format bar
// Inputs		: None
// Output		: Changes the font in local copy of preferences
//				  Applies the new font to the Typed text box
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnFont() 
{
	CFontDialog 	fontDialog;
	LOGFONT font;

	// If user chose a font
	if(fontDialog.DoModal() == IDOK)
	{
		// Find out the font user chose
		fontDialog.GetCurrentFont(&font);

		// If user did not choose any font, no need to process it
		if(!strcmp(font.lfFaceName, ""))
			return;

		// Get font color
		m_Preferences.m_iFontColor = fontDialog.GetColor();

		// Get font size
		m_Preferences.m_iFontSize = fontDialog.GetSize() * 2;

		// Put this font into preferences
		m_Preferences.m_Font = font;

		// Apply these settings to the Typed Text box
		SetTextFont(font, m_Preferences.m_strFontFace, m_ctrlTypedText, m_Preferences.m_iFontColor, m_Preferences.m_iFontSize, FORMAT_ALL_TEXT);
	}
}



/**************************************************************************/
// Method Name	: OnBackcolor
// Description	: Calld when User presses Color button on format bar
// Inputs		: None
// Output		: Applies the new background color to the text boxes
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnBackcolor() 
{
	CColorDialog 	colorDialog;

	// If user chose a color
	if(colorDialog.DoModal() == IDOK)
	{
		// Find out the color user chose
		m_Preferences.m_iColor = colorDialog.GetColor();

		// Set this color as background color for the speak window
		SetControlsBackColor(m_Preferences.m_iColor);
	}
}



/**************************************************************************/
// Method Name	: OnFileNew
// Description	: Calld when User clicks File-New menu item
// Inputs		: None
// Output		: Creates a new speak window with no To name
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnFileNew() 
{
	CString str = "";
	m_pView->CreateASpeakWindow(str, NULL, str, &(m_Preferences.m_Font), m_Preferences.m_iFontColor, m_Preferences.m_iFontSize);
}



/**************************************************************************/
// Method Name	: OnFileSend
// Description	: Calld when User clicks File-Send menu item
// Inputs		: None
// Output		: Same as when Send button is pressed
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnFileSend() 
{
	OnSend();
}



/**************************************************************************/
// Method Name	: OnFileClose
// Description	: Calld when User clicks File-Close menu item
// Inputs		: None
// Output		: Same as when window is closed
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnFileClose() 
{
	m_pFrame->DestroyWindow();
}



/**************************************************************************/
// Method Name	: OnFormatFont
// Description	: Calld when User clicks Format-Font menu item
// Inputs		: None
// Output		: Same as when Font button is pressed
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnFormatFont() 
{
	OnFont();
}



/**************************************************************************/
// Method Name	: OnFormatColor
// Description	: Calld when User clicks Format-Color menu item
// Inputs		: None
// Output		: Same as when Color button is pressed
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnFormatColor() 
{
	OnBackcolor();
}



/**************************************************************************/
// Method Name	: OnFormatDefaultfontandcolor
// Description	: Calld when User clicks Format-Set Default Font & Color menu item
// Inputs		: None
// Output		: Sets the font and background colot to default
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnFormatDefaultfontandcolor() 
{
	// Overwrite the window settings by default setting
	m_Preferences = m_pParent->m_Preferences;

	// Set the new font to the types text box. Keep the
	// existing text intact
	SetTextFont(m_Preferences.m_Font, m_Preferences.m_strFontFace, m_ctrlTypedText, m_Preferences.m_iFontColor, m_Preferences.m_iFontSize, FORMAT_APPEND_TEXT);

	// Set the background color by default
	SetControlsBackColor(m_Preferences.m_iColor);
}



/**************************************************************************/
// Method Name	: SetControlsBackColor
// Description	: Used to set background colors for rich text controls
// Inputs		: None
// Output		: Sets the background color of both rich edit controls
//					to the given color
// Returns		: void
/**************************************************************************/
void CSpeakWindow::SetControlsBackColor(int iColor)
{
	m_ctrlTypedText.SetBackgroundColor(FALSE, iColor);
	m_ctrlWholeText.SetBackgroundColor(FALSE, iColor);
}



/**************************************************************************/
// Method Name	: OnEditCut
// Description	: Calld when User clicks Edit-Cut menu item
// Inputs		: None
// Output		: Cuts the selected text from typed text or whole text box
//					and puts it onto clipboard
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnEditCut() 
{
	// Check if some text is selected from typed text
	CString selText = m_ctrlTypedText.GetSelText();

	if(selText.GetLength())			// If some text is selected here
	{
		// Put this text onto clipboard and return
		m_ctrlTypedText.Cut();
		return;
	}

	// Check if some text is selected from whole text
	selText = m_ctrlWholeText.GetSelText();

	if(selText.GetLength())			// If some text is selected here
	{
		// Put this text onto clipboard and return
		// Do not try to cut this, as whole text box is read only
		m_ctrlWholeText.Copy();
	}
}



/**************************************************************************/
// Method Name	: OnEditCopy
// Description	: Calld when User clicks Edit-Copy menu item
// Inputs		: None
// Output		: Copies the selected text from typed text or whole text box
//					to the clipboard
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnEditCopy() 
{
	// Check if some text is selected from typed text
	CString selText = m_ctrlTypedText.GetSelText();

	if(selText.GetLength())			// If some text is selected here
	{
		// Put this text onto clipboard and return
		m_ctrlTypedText.Copy();
		return;
	}

	// Check if some text is selected from whole text
	selText = m_ctrlWholeText.GetSelText();

	if(selText.GetLength())			// If some text is selected here
	{
		// Put this text onto clipboard and return
		// Do not try to cut this, as whole text box is read only
		m_ctrlWholeText.Copy();
	}
}



/**************************************************************************/
// Method Name	: OnEditPaste
// Description	: Calld when User clicks Edit-Paste menu item
// Inputs		: None
// Output		: If text on the clipboard is text / rich text, and active 
//					window is Typed text box, pastes the text onto typed text
//					box
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnEditPaste() 
{
	// If active focus is on typed text box
	if(GetFocus() == &m_ctrlTypedText)
		m_ctrlTypedText.Paste();
}



/**************************************************************************/
// Method Name	: OnEditDelete
// Description	: Calld when User clicks Edit-Delete menu item
// Inputs		: None
// Output		: If active window is Typed text box, deletes the selected text
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnEditDelete() 
{
	// If active focus is on typed text box
	if(GetFocus() == &m_ctrlTypedText)
	{
		// Check if some text is selected from typed text
		CString selText = m_ctrlTypedText.GetSelText();

		if(selText.GetLength())			// If some text is selected here
			m_ctrlTypedText.ReplaceSel("");		// Delete it
	}
}



/**************************************************************************/
// Method Name	: OnEditSelectall
// Description	: Calld when User clicks Edit-Select All menu item
// Inputs		: None
// Output		: If active window is any text box, selects entire text in 
//					that box
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnEditSelectall() 
{
	if(GetFocus() == &m_ctrlTypedText)
	{
		m_ctrlTypedText.SetSel(0, -1);
		return;
	}
		
	if(GetFocus() == &m_ctrlWholeText)
		m_ctrlWholeText.SetSel(0, -1);
}



/**************************************************************************/
// Method Name	: OnFileSave
// Description	: Calld when User clicks File-Save menu item
// Inputs		: None
// Output		: Checks whether there's anything to save in text box
//				  Checks whether anything has changed since last save
//				  Calls DoSaving if file is saved at least once
//				  Calls OnFileSaveAs otherwise
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnFileSave() 
{
	// If there's nothing in whole text box, no need 2 go further
	if(m_strWholeText.GetLength() == 0)
	{
		::AfxMessageBox("Nothing to save!");
		return;
	}

	// If file is not changed, no need to go further
	if(!m_bDirty)
		return;

	// If file is not saved even once, we need File - Save as functionality
	if(!m_bSavedOnce)
		OnFileSaveas();
	// If file is saved, we just need to re write it with current contents
	else
		DoSaving();
}



/**************************************************************************/
// Method Name	: DoSaving
// Description	: Common functionality for Save and Save As menu items
// Inputs		: None
// Output		: Overwrites the file with contents of Whole Text box
// Returns		: void
/**************************************************************************/
void CSpeakWindow::DoSaving()
{
	// Open file in write mode
	ofstream fileHandle;
	fileHandle.open(m_strFileName.GetBuffer(0));

	// If opening failed
	if(!fileHandle)
	{
		AfxMessageBox("File could not be written. Please check permissions.");
		return;
	}

	// Write the file
	char *pch = m_strWholeText.GetBuffer(0);
	fileHandle.write(pch, m_strWholeText.GetLength());

	// Close the file handle
	fileHandle.close();

	// Set the saved flag
	m_bSavedOnce = true;	
}



/**************************************************************************/
// Method Name	: OnFileSaveas
// Description	: Calld when User clicks File-Save As menu item
// Inputs		: None
// Output		: Gets file name and location from user via a file dialog
//				  If file exists, warns abt overwrite
//				  Calls DoSaving
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnFileSaveas() 
{
	CFileDialog dlg(FALSE, "txt", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "*.txt");

	// Get file name
	if(dlg.DoModal() == IDOK)
	{
		CString filename = dlg.GetFileName();

		// Write the file
		m_strFileName = filename;
		DoSaving();
	}
}



/**************************************************************************/
// Method Name	: OnFileBlockthisuser
// Description	: Called when user clicks Block this User menu item or toolbar button
// Inputs		: None
// Output		: Asks the user whether he wants to permanently block this sender
//				  Sends a message to the server, requesting this user to be blocked
// Returns		: void
/**************************************************************************/
void CSpeakWindow::OnFileBlockthisuser() 
{
	// Create blocked user confirmation dialog, and show in modal fashion
	CBlockedUserConfirmDlg dlg;
	int answer = dlg.DoModal();

	// Ask user if he wants to permanently block the sender or just for this session
	//int answer = ::AfxMessageBox("Do you want to permanently block this user?", MB_YESNOCANCEL);

	// If user pressed Cancel, no need to go further
	if(answer == IDCANCEL)
		return;

	// Enter this name into blocked users list here
	CMessagePacket *msgPacket = new CMessagePacket();
	msgPacket->m_strFrom = m_strTo;
	if(answer == IDNO)
		msgPacket->m_strFrom += "*";

	// Append a '*' here if this user is being blocked only for this session
	m_pParent->m_BlockedUsersList.AddTail(msgPacket);

	// Create a msg packet
	CMessagePacket msg;
	msg.m_strFrom = m_pParent->m_UserName;
	msg.m_strTo = m_strTo;
	
	if(answer == IDYES)		//	Permanently block this To user
		msg.m_strReasonFlag = "BUP";
	else					// Block only for this session
		msg.m_strReasonFlag = "BU";

	m_pParent->SendMessagePacket(msg);
}
