// TextPane.cpp : implementation file
//

#include "stdafx.h"
#include "MessengerClient.h"
#include "TextPane.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTextPane

IMPLEMENT_DYNCREATE(CTextPane, CFormView)

CTextPane::CTextPane()
	: CFormView(CTextPane::IDD)
{
	//{{AFX_DATA_INIT(CTextPane)
	m_strMessageBody = _T("");
	//}}AFX_DATA_INIT
	m_bFirstTime = true;
}

CTextPane::~CTextPane()
{
}

void CTextPane::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTextPane)
	DDX_Control(pDX, IDC_RICHEDIT_MESSAGEBODY, m_ctrlMessageBody);
	DDX_Control(pDX, IDC_DELETE, m_btnDelete);
	DDX_Control(pDX, IDC_REPLY, m_btnReply);
	DDX_Text(pDX, IDC_RICHEDIT_MESSAGEBODY, m_strMessageBody);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTextPane, CFormView)
	//{{AFX_MSG_MAP(CTextPane)
	ON_BN_CLICKED(IDC_DELETE, OnDelete)
	ON_BN_CLICKED(IDC_REPLY, OnReply)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTextPane diagnostics

#ifdef _DEBUG
void CTextPane::AssertValid() const
{
	CFormView::AssertValid();
}

void CTextPane::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTextPane message handlers

/**************************************************************************/
// Method Name	: OnFileDelete()
// Description	: Interface for OnDelete method
// Inputs		: none
// Output		: If any message is selected, deletes it from list
//				  Deletes that message from offline messages list in memory
// Returns		: void
/**************************************************************************/
void CTextPane::OnFileDelete() 
{
	OnDelete();
}


/**************************************************************************/
// Method Name	: OnFileReply()
// Description	: Interface for OnReply method
// Inputs		: none
// Output		: Creates speak windows for the selected message's sender
// Returns		: void
/**************************************************************************/
void CTextPane::OnFileReply() 
{
	OnReply();
}



/**************************************************************************/
// Method Name	: GetViewPtr()
// Description	: This method is for getting the view object's pointer
// Inputs		: none
// Output		: Get the View object's pointer. This is same as GetViewPtr()
//				  method is Doc class. So change it when you change the doc
//				  class method, and vice versa
// Returns		: View class pointer (CMessengerClientView *)
/**************************************************************************/
CMessengerClientView * CTextPane::GetViewPtr()
{
	CMainFrame *wnd = (CMainFrame *)::AfxGetMainWnd();
	CMDIChildWnd *child = (CMDIChildWnd *)wnd->GetActiveFrame();

	CMessengerClientView *pView = (CMessengerClientView *)child->GetActiveView();

	return pView;
}



/**************************************************************************/
// Method Name	: OnDelete()
// Description	: This method is called when user clicks the Delete button
// Inputs		: none
// Output		: If any message is selected, deletes it from list
//				  Deletes that message from offline messages list in memory
// Returns		: void
/**************************************************************************/
void CTextPane::OnDelete() 
{
	// First check if any of the messages is selected or not
	if(!m_pMsgsListPane->m_lstMsgs.GetSelectedCount())
		return;

	// See if any of the list members is selected
	POSITION pos = m_pMsgsListPane->m_lstMsgs.GetFirstSelectedItemPosition();
	if(!pos)
		return;

	// Delete the selected item from list
	int index = m_pMsgsListPane->m_lstMsgs.GetSelectionMark();
	m_pMsgsListPane->m_lstMsgs.DeleteItem(index);

	// Reflect these changes in this pane as well
	m_strMessageBody = "";
	UpdateData(FALSE);

	// Get the view pointer
	CMessengerClientView *pView = GetViewPtr();
	CMessengerClientDoc *pDoc = pView->GetDocument();

	// Delete this indexed item from offline messages list
	// store in memory
	pos = pDoc->m_OfflineMessagesList.FindIndex(index);
	pDoc->m_OfflineMessagesList.RemoveAt(pos);

	// Update this pane
	m_pMsgsListPane->UpdateTextButtonsPane();
}



/**************************************************************************/
// Method Name	: OnReply()
// Description	: This method is called when user clicks the Reply button
// Inputs		: none
// Output		: Creates speak window for the selected message's sender
// Returns		: void
/**************************************************************************/
void CTextPane::OnReply() 
{
	// First check if any of the messages is selected or not
	if(!m_pMsgsListPane->m_lstMsgs.GetSelectedCount())
		return;

	m_pMsgsListPane->CallDoubleClickMessagesList();
}



/**************************************************************************/
// Method Name	: AdjustFormControls
// Description	: Method to put all controls on the form in their proper places
//				  Called from various places
// Inputs		: void
// Output		: All controls on the form resized and adjusted
// Returns		: void
/**************************************************************************/
void CTextPane::AdjustFormControls()
{
	// These rects will contain the coordinates of this pane
	// and that of various controls
	CRect rectWin, rectTextBox, rectReplyBtn, rectDeleteBtn;

	// Get main window's rects
	GetWindowRect(&rectWin);

	/****************************************************************************/
	// Position the List control
	/****************************************************************************/
	rectTextBox = rectWin;
	rectTextBox.bottom -= (BTN_HEIGHT + (2 * BUFFER_DISTANCE));

	ScreenToClient(&rectTextBox);
	m_ctrlMessageBody.MoveWindow(rectTextBox.left, rectTextBox.top,
									rectTextBox.Width(), rectTextBox.Height());
	
	/****************************************************************************/
	// Position the Delete button
	/****************************************************************************/
	rectDeleteBtn.top = rectTextBox.bottom + BUFFER_DISTANCE;
	rectDeleteBtn.bottom = rectDeleteBtn.top + BTN_HEIGHT;
	rectDeleteBtn.left = rectTextBox.left + BUFFER_DISTANCE;
	rectDeleteBtn.right = rectDeleteBtn.left + BTN_WIDTH;
	m_btnDelete.MoveWindow(rectDeleteBtn.left, rectDeleteBtn.top,
									rectDeleteBtn.Width(), rectDeleteBtn.Height());

	/****************************************************************************/
	// Position the Reply button
	/****************************************************************************/
	rectReplyBtn = rectDeleteBtn;
	rectReplyBtn.left = rectDeleteBtn.right + BUFFER_DISTANCE;
	rectReplyBtn.right = rectReplyBtn.left + BTN_WIDTH;
	m_btnReply.MoveWindow(rectReplyBtn.left, rectReplyBtn.top,
									rectReplyBtn.Width(), rectReplyBtn.Height());

	m_bFirstTime = false;
}



/**************************************************************************/
// Method Name	: OnSize
// Description	: Called when window resizes
// Inputs		: New window coordinates, system defined
// Output		: Calls the base class method
//				  If it's the first time window is being resized, calls method
//				  to resize and adjust controls
// Returns		: void
/**************************************************************************/
void CTextPane::OnSize(UINT nType, int cx, int cy) 
{
	CFormView::OnSize(nType, cx, cy);

	// Do this only if the window is resized after display
	if(!m_bFirstTime)
		// Resize controls appropriately and place them on proper positions
		AdjustFormControls();
	else
		m_bFirstTime = false;
}
